@@ -27,9 +27,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 21851 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 01:35:12 +0000 (Mon, 22 Dec 2008) $
+ * $Revision: 22097 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
  */
 
 #ifndef OPAL_OPAL_CONNECTION_H
@@ -451,14 +451,15 @@ class OpalConnection : public PSafeObject
        instances. These phases are fully described in the documentation page
        \ref pageOpalConnections.  */
     enum Phases {
-	UninitialisedPhase,   /*!< Indicates the OpalConnection instance has just been constructed*/
-	SetUpPhase,           /*!< In the process of sending/receiving the initial INVITE packet */
-	AlertingPhase,        /*!< There is a phone ringing, somewhere                           */
-	ConnectedPhase,       /*!< There is agreement on having a call, media type etc            */
-	EstablishedPhase,     /*!< Media is flowing, control streams  are all operational         */
-	ReleasingPhase,       /*!< Hangup packet has been sent/received, media and control not yet stopped  */
-	ReleasedPhase,        /*!< Media and control streams have been terminated                           */
-	NumPhases             /*!< Number of available phases. Can be used to indicate an unknown phase     */
+      UninitialisedPhase,   //!< Indicates the OpalConnection instance has just been constructed
+      SetUpPhase,           //!< In the process of sending/receiving the initial INVITE packet
+      ProceedingPhase,      //!< The remote is now responsible for completing the call
+      AlertingPhase,        //!< The remote says there is a phone ringing, somewhere
+      ConnectedPhase,       //!< There is agreement on having a call, usually means billing will apply
+      EstablishedPhase,     //!< Media is flowing, control streams  are all operational
+      ReleasingPhase,       //!< Hangup packet has been sent/received, media and control not yet stopped
+      ReleasedPhase,        //!< Media and control streams have been terminated
+      NumPhases             //!< Number of available phases. Can be used to indicate an unknown phase
     };
 
     /**Get the phase of the connection.
@@ -591,18 +592,29 @@ class OpalConnection : public PSafeObject
     virtual PBoolean OnSetUpConnection();
 
     
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour calls the OpalEndPoint function of the same name.
+     */
+    virtual void OnProceeding();
+
     /**Call back for remote party being alerted.
        This function is called after the connection is informed that the
        remote endpoint is "ringing". Generally some time after the
        SetUpConnection() function was called, this is function is called.
 
-       If PFalse is returned the connection is aborted.
-
        If an application overrides this function, it should generally call the
-       ancestor version for correct operation. An application would typically
-       only intercept this function if it wishes to do some form of logging.
-       For this you can obtain the name of the caller by using the function
-       OpalConnection::GetRemotePartyName().
+       ancestor version for correct operation.
 
        The default behaviour calls the OpalEndPoint function of the same name.
      */
@@ -1291,6 +1303,38 @@ class OpalConnection : public PSafeObject
      */
     void CopyPartyNames(const OpalConnection & other);
 
+
+    /**Get alerting type information of an incoming call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns an empty string.
+      */
+    virtual PString GetAlertingType() const;
+
+    /**Set alerting type information for outgoing call.
+       The type of "distinctive ringing" for the call. The string is protocol
+       dependent, so the caller would need to be aware of the type of call
+       being made. Some protocols may ignore the field completely.
+
+       For SIP this corresponds to the string contained in the "Alert-Info"
+       header field of the INVITE. This is typically a URI for the ring file.
+
+       For H.323 this must be a string representation of an integer from 0 to 7
+       which will be contained in the Q.931 SIGNAL (0x34) Information Element.
+
+       Default behaviour returns false.
+      */
+    virtual bool SetAlertingType(const PString & info);
+
+
     /**Get the default maximum audio jitter delay parameter.
        Defaults to 50ms
      */
@@ -1317,9 +1361,9 @@ class OpalConnection : public PSafeObject
     OpalEchoCanceler * GetEchoCanceler() const { return echoCanceler; }
 
     /**Get the protocol-specific unique identifier for this connection.
+       Default behaviour just returns the connection token.
      */
-    virtual const OpalGloballyUniqueID & GetIdentifier() const
-    { return callIdentifier; }
+    virtual PString GetIdentifier() const;
 
     /**Get the maximum RTP payload size. This function allows a user to
        override the value returned on a connection by connection basis, for
@@ -1421,7 +1465,6 @@ class OpalConnection : public PSafeObject
 
   protected:
     PString              callToken;
-    OpalGloballyUniqueID callIdentifier;
     PBoolean             originating;
     PTime                setupTime;
     PTime                alertingTime;
