@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22089 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-19 04:19:55 +0000 (Thu, 19 Feb 2009) $
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -540,7 +540,7 @@ class OpalRTPMediaStream : public OpalMediaStream
 
 
 
-/**This class describes a media stream that transfers data to/from a PChannel.
+/**This class describes a media stream that transfers PCM-16 data to/from a PChannel.
   */
 class OpalRawMediaStream : public OpalMediaStream
 {
@@ -587,8 +587,15 @@ class OpalRawMediaStream : public OpalMediaStream
 
     /**Return the associated PChannel 
      */
-    PChannel * GetChannel() { return channel; }
-    
+    PChannel * GetChannel() { return m_channel; }
+
+    /**Set a new channel for raw PCM stream.
+      */
+    bool SetChannel(
+      PChannel * channel,     ///< New channel
+      bool autoDelete = true  ///< Auto delete channel on exit or replacement
+    );
+
     /**Close the media stream.
 
        Closes the associated PChannel.
@@ -601,12 +608,15 @@ class OpalRawMediaStream : public OpalMediaStream
   //@}
 
   protected:
-    PChannel * channel;
-    bool       autoDelete;
+    PChannel * m_channel;
+    bool       m_autoDelete;
+    PMutex     m_channelMutex;
 
-    PUInt64    averageSignalSum;
-    unsigned   averageSignalSamples;
-    PMutex     averagingMutex;
+    PBYTEArray m_silence;
+
+    PUInt64    m_averageSignalSum;
+    unsigned   m_averageSignalSamples;
+    PMutex     m_averagingMutex;
 
     void CollectAverage(const BYTE * buffer, PINDEX size);
 };
