@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
- * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Revision: 21391 $
+ * $Author: csoutheren $
+ * $Date: 2008-10-21 04:55:41 +0000 (Tue, 21 Oct 2008) $
  */
 
-#ifndef __OPAL_PATCH_H
-#define __OPAL_PATCH_H
+#ifndef OPAL_OPAL_PATCH_H
+#define OPAL_OPAL_PATCH_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -40,13 +40,12 @@
 
 #include <opal/mediastrm.h>
 #include <opal/mediacmd.h>
+#include <codec/ratectl.h>
 
 #include <list>
 
-
 class OpalTranscoder;
 
-
 /**Media stream "patch cord".
    This class is the thread of control that transfers data from one
    "source" OpalMediStream to one or more other "sink" OpalMediStream
@@ -94,6 +93,16 @@ class OpalMediaPatch : public PObject
       */
     virtual void Start();
 
+    /**Indicate the patch has started. Typically called from the beginning
+       of the patch thread.
+
+       Default behaviour make sure jitter buffer is created.
+
+       Returns bool indicating all media streams are asynchronous so the patch
+               should assure no thread startvation
+      */
+    virtual bool OnPatchStart();
+
     /**Close the patch.
       This is an internal function that closes all of the sink streams and
        waits for the the thread to terminate. It is called when the source
@@ -161,9 +170,8 @@ class OpalMediaPatch : public PObject
        The default behaviour updates the source/sink media stream and the
        output side of any relevant transcoders.
       */
-    virtual PBoolean UpdateMediaFormat(
-      const OpalMediaFormat & mediaFormat,  ///<  New media format
-      PBoolean fromSink                         ///<  Flag for source or sink
+    virtual bool UpdateMediaFormat(
+      const OpalMediaFormat & mediaFormat  ///<  New media format
     );
 
     /**Execute the command specified to the transcoder. The commands are
@@ -233,21 +241,10 @@ class OpalMediaPatch : public PObject
 
 #if OPAL_VIDEO
         void SetRateControlParameters(const OpalMediaFormat & mediaFormat);
-        bool RateControlExceeded(const PTimeInterval & currentTime);
+        bool RateControlExceeded();
 
-        bool          rcEnabled;
-        unsigned      rcByteRate;
-        unsigned      rcWindowSize;
-        unsigned      rcMaxConsecutiveFramesSkip;
-        unsigned      rcConsecutiveFramesSkipped;
-        PTimeInterval rcLastTime;
-        PINDEX        rcTotalSize;
-        
-        struct FrameInfo {
-          PTimeInterval time;
-          PINDEX        size;
-        };
-        std::list<FrameInfo> frameInfoList;
+        bool rcEnabled;
+        OpalVideoRateController rateController;
 #endif
     };
     PList<Sink> sinks;
@@ -295,7 +292,7 @@ class OpalPassiveMediaPatch : public OpalMediaPatch
 };
 
 
-#endif // __OPAL_PATCH_H
+#endif // OPAL_OPAL_PATCH_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
