@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 21986 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2009-02-04 04:31:07 +0000 (Wed, 04 Feb 2009) $
  */
 
 #ifndef OPAL_RTP_RTP_H
@@ -46,13 +46,6 @@ class RTP_JitterBuffer;
 class PNatMethod;
 class OpalSecurityMode;
 
-#if OPAL_RTP_AGGREGATE
-#include <ptclib/sockagg.h>
-#else
-typedef void * PHandleAggregator;
-typedef void * RTP_AggregatedHandle;
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////
 // 
 // class to hold the QoS definitions for an RTP channel
@@ -75,7 +68,7 @@ class RTP_DataFrame : public PBYTEArray
   PCLASSINFO(RTP_DataFrame, PBYTEArray);
 
   public:
-    RTP_DataFrame(PINDEX payloadSize = 2048);
+    RTP_DataFrame(PINDEX payloadSize, PINDEX bufferSize = 0);
     RTP_DataFrame(const BYTE * data, PINDEX len, PBoolean dynamic = PTrue);
 
     enum {
@@ -369,11 +362,11 @@ class RTP_UserData : public PObject
     ) const;
 #endif
 
-    /**Callback from the RTP session when a call needs to be shut down.
+    /**Callback from the RTP session when RTP session is failing due to the remote being unavailable
        The default behaviour does nothing.
       */
-    virtual void OnClearCall(
-      const RTP_Session & session   ///<  Session with statistics
+    virtual void SessionFailing(
+      RTP_Session & session   ///<  Session with statistics
     );
 };
 
@@ -392,9 +385,6 @@ class RTP_Session : public PObject
     struct Params {
       Params()
         : id(0)
-#if OPAL_RTP_AGGREGATE
-        , aggregator(NULL)
-#endif
         , userData(NULL)
         , autoDelete(true)
         , isAudio(false)
@@ -403,9 +393,6 @@ class RTP_Session : public PObject
 
       PString             encoding;    ///<  identifies initial RTP encoding (RTP/AVP, UDPTL etc)
       unsigned            id;          ///<  Session ID for RTP channel
-#if OPAL_RTP_AGGREGATE
-      PHandleAggregator * aggregator;  ///<  RTP aggregator
-#endif
       RTP_UserData      * userData;    ///<  Optional data for session.
       bool                autoDelete;  ///<  Delete optional data with session.
       bool                isAudio;     ///<  is audio RTP data
@@ -697,6 +684,10 @@ class RTP_Session : public PObject
       unsigned packets   ///<  Number of packets between callbacks
     );
 
+    /**Clear statistics
+      */
+    void ClearStatistics();
+
     /**Get total number of packets sent in session.
       */
     DWORD GetPacketsSent() const { return packetsSent; }
@@ -827,6 +818,14 @@ class RTP_Session : public PObject
 
     friend class EncodingLock; 
 
+    void SetFailed(bool v)
+    { failed = v; }
+
+    bool HasFailed() const
+    { return failed; }
+
+    void AddFilter(const PNotifier & filter);
+
   protected:
     virtual void SendBYE();
     void AddReceiverReport(RTP_ControlFrame::ReceiverReport & receiver);
@@ -906,12 +905,17 @@ class RTP_Session : public PObject
     PMutex reportMutex;
     PTimer reportTimer;
 
-#if OPAL_RTP_AGGREGATE
-    PHandleAggregator * aggregator;
-#endif
-
     PBoolean closeOnBye;
     PBoolean byeSent;
+    bool                failed;      ///<  set to true if session has received too many ICMP destination unreachable
+
+    class Filter : public PObject {
+        PCLASSINFO(Filter, PObject);
+      public:
+        Filter(const PNotifier & n) : notifier(n) { }
+        PNotifier notifier;
+    };
+    PList<Filter> filters;
 };
 
 /**This class is for the IETF Real Time Protocol interface on UDP/IP.
@@ -1099,6 +1103,7 @@ class RTP_UDP : public RTP_Session
     bool localHasNAT;
     bool first;
     int  badTransmitCounter;
+    PTime badTransmitStart;
 };
 
 /////////////////////////////////////////////////////////////////////////////
