@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 21936 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2009-01-17 08:02:24 +0000 (Sat, 17 Jan 2009) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -60,7 +60,7 @@ class MyManager;
 
 class OpalLineEndPoint;
 class OpalIVREndPoint;
-class OpalT38EndPoint;
+class OpalFaxEndPoint;
 
 class wxSplitterWindow;
 class wxSplitterEvent;
@@ -69,6 +69,7 @@ class wxListCtrl;
 class wxListEvent;
 class wxNotebook;
 class wxGrid;
+class wxConfigBase;
 
 
 class PwxString : public wxString
@@ -109,10 +110,21 @@ class PwxString : public wxString
     inline bool operator!=(const PwxString & other)       const { return !IsSameAs(other); }
     inline bool operator!=(const OpalMediaFormat & other) const { return !IsSameAs(wxString(other, wxConvUTF8)); }
 
+#if wxUSE_UNICODE
+    inline PString p_str() const { return ToUTF8().data(); }
+    inline operator PString() const { return ToUTF8().data(); }
+    inline operator PFilePath() const { return ToUTF8().data(); }
+    inline operator PIPSocket::Address() const { return PString(ToUTF8().data()); }
+    inline friend ostream & operator<<(ostream & stream, const PwxString & string) { return stream << string.ToUTF8(); }
+    inline friend wostream & operator<<(wostream & stream, const PwxString & string) { return stream << string.c_str(); }
+#else
+    inline PString p_str() const { return c_str(); }
     inline operator PString() const { return c_str(); }
-    inline operator PFilePath() const { return PFilePath(c_str()); }
-    inline operator PIPSocket::Address() const { return PIPSocket::Address(PString(c_str())); }
+    inline operator PFilePath() const { return c_str(); }
+    inline operator PIPSocket::Address() const { return c_str(); }
     inline friend ostream & operator<<(ostream & stream, const PwxString & string) { return stream << string.c_str(); }
+    inline friend wostream & operator<<(wostream & stream, const PwxString & string) { return stream << string.c_str(); }
+#endif
 };
 
 
@@ -130,6 +142,20 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
     MyManager & m_manager;
 };
 
+class IMDialog;
+
+struct MessageInfo {
+  PString callId;
+  SIPURL from;
+  PString remoteContact;
+  PString body;
+};
+
+struct PresenceInfo {
+  PString entity;
+  PString status;
+};
+
 #if OPAL_H323
 class MyH323EndPoint : public H323EndPoint
 {
@@ -164,12 +190,48 @@ class MySIPEndPoint : public SIPEndPoint
       bool reSubscribing,
       SIP_PDU::StatusCodes reason
     );
+    virtual void OnDialogInfoReceived(
+      const SIPDialogNotification & info  ///< Information on dialog state change
+    );
+    virtual void OnMessageReceived(const SIPURL & from, const SIP_PDU & pdu);
+    virtual void OnPresenceInfoReceived(const SIPPresenceInfo & info);
 
     MyManager & m_manager;
 };
 #endif // OPAL_SIP
 
 
+class PresenceDialog : public wxDialog
+{
+  public:
+    PresenceDialog(MyManager * manager, SIPEndPoint & sipEP);
+
+  private:
+    bool TransferDataFromWindow();
+
+    SIPEndPoint & m_sipEP;
+    PwxString     m_address;
+    PwxString     m_status;
+
+    DECLARE_EVENT_TABLE()
+};
+
+
+class AudioDevicesDialog : public wxDialog
+{
+  public:
+    AudioDevicesDialog(MyManager * manager, const OpalPCSSConnection & connection);
+
+    PString GetTransferAddress() const;
+
+  private:
+    PwxString     m_playDevice;
+    PwxString     m_recordDevice;
+
+    DECLARE_EVENT_TABLE()
+};
+
+
 class VideoControlDialog : public wxDialog
 {
   public:
@@ -188,10 +250,11 @@ class VideoControlDialog : public wxDialog
 class CallDialog : public wxDialog
 {
   public:
-    CallDialog(MyManager * manager, bool hideHandset);
+    CallDialog(MyManager * manager, bool hideHandset, bool hideFax);
 
     PwxString m_Address;
     bool      m_UseHandset;
+    int       m_FaxMode;
 
   private:
     void OnOK(wxCommandEvent & event);
@@ -203,25 +266,93 @@ class CallDialog : public wxDialog
     DECLARE_EVENT_TABLE()
 };
 
-
-class CallingPanel : public wxPanel
+class CallIMDialog : public wxDialog
 {
   public:
-    CallingPanel(MyManager & manager, wxWindow * parent);
+    CallIMDialog(MyManager * manager);
+
+    PwxString m_Address;
 
   private:
-    void OnHangUp(wxCommandEvent & event);
+    void OnOK(int code);
+    void OnPage(wxCommandEvent & event);
+    void OnSession(wxCommandEvent & event);
+    void OnAddressChange(wxCommandEvent & event);
 
-    MyManager & m_manager;
+    wxComboBox * m_AddressCtrl;
+    wxButton   * m_ok;
+
+    DECLARE_EVENT_TABLE()
+};
+
+class IMDialog : public wxDialog
+{
+  public:
+    IMDialog(MyManager * manager, const PString & callId, const SIPURL & them, const PString & remoteContact);
+    ~IMDialog();
+
+    void AddTextToScreen(const wxString & text, bool fromUs);
+
+    PString   m_callId;
+    PString   m_remoteContact;
+
+  private:
+    void OnSend(wxCommandEvent & event);
+    void OnTextEnter(wxCommandEvent & event);
+    void OnCloseWindow(wxCloseEvent &event);
+
+    void SendCurrentText();
+
+    MyManager * m_manager;
+
+    PwxString m_them;
+    PwxString m_us;
+
+    wxTextCtrl * m_enteredText;
+    wxTextCtrl * m_textArea;
+    wxTextAttr   m_defaultStyle;
+    wxTextAttr   m_ourStyle;
+    wxTextAttr   m_theirStyle;
 
     DECLARE_EVENT_TABLE()
 };
 
 
-class AnswerPanel : public wxPanel
+class CallPanelBase : public wxPanel
+{
+  protected:
+    CallPanelBase(
+      MyManager & manager,
+      const PwxString & token,
+      wxWindow * parent,
+      const wxChar * resource
+    );
+
+  public:
+    const PwxString & GetToken() const { return m_token; }
+
+  protected:
+    MyManager & m_manager;
+    PwxString   m_token;
+};
+
+
+class CallingPanel : public CallPanelBase
 {
   public:
-    AnswerPanel(MyManager & manager, wxWindow * parent);
+    CallingPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
+
+  private:
+    void OnHangUp(wxCommandEvent & event);
+
+    DECLARE_EVENT_TABLE()
+};
+
+
+class AnswerPanel : public CallPanelBase
+{
+  public:
+    AnswerPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
 
     void SetPartyNames(const PwxString & calling, const PwxString & called);
 
@@ -230,8 +361,6 @@ class AnswerPanel : public wxPanel
     void OnReject(wxCommandEvent & event);
     void OnChangeAnswerMode(wxCommandEvent & event);
 
-    MyManager & m_manager;
-
     DECLARE_EVENT_TABLE()
 };
 
@@ -252,6 +381,7 @@ enum StatisticsPages {
 struct StatisticsField
 {
   StatisticsField(const wxChar * name, StatisticsPages page);
+  virtual ~StatisticsField() { }
   void Init(wxWindow * panel);
   void Clear();
   double CalculateBandwidth(DWORD bytes);
@@ -301,10 +431,10 @@ class StatisticsPage
     void operator=(const StatisticsPage &) { }
 };
 
-class InCallPanel : public wxPanel
+class InCallPanel : public CallPanelBase
 {
   public:
-    InCallPanel(MyManager & manager, wxWindow * parent);
+    InCallPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
     virtual bool Show(bool show = true);
 
     void OnStreamsChanged();
@@ -312,7 +442,7 @@ class InCallPanel : public wxPanel
 
   private:
     void OnHangUp(wxCommandEvent & event);
-    void OnRequestHold(wxCommandEvent & event);
+    void OnHoldRetrieve(wxCommandEvent & event);
     void OnSpeakerMute(wxCommandEvent & event);
     void OnMicrophoneMute(wxCommandEvent & event);
     void OnUserInput1(wxCommandEvent & event);
@@ -336,7 +466,6 @@ class InCallPanel : public wxPanel
 
     bool GetConnections(bool user);
 
-    MyManager & m_manager;
     wxButton  * m_Hold;
     wxButton  * m_SpeakerHandset;
     wxCheckBox* m_SpeakerMute;
@@ -364,6 +493,7 @@ class SpeedDialDialog : public wxDialog
     wxString m_Name;
     wxString m_Number;
     wxString m_Address;
+    wxString m_StateURL;
     wxString m_Description;
 
   private:
@@ -382,42 +512,58 @@ class SpeedDialDialog : public wxDialog
 };
 
 
-class RegistrarInfo
+class RegistrationInfo
 {
   public:
-    RegistrarInfo()
-      : m_Active(false)
-      , m_TimeToLive(300)
-      , m_MWI(true)
-      , m_Presence(true)
-    {
-    }
+    enum Types {
+      Register,
+      SubscribeMWI,
+      SusbcribePresence,
+      SubscribeMLA,
+      PublishPresence
+    };
 
-    bool operator==(const RegistrarInfo & other) const
-    {
-      return m_Active     == other.m_Active &&
-             m_User       == other.m_User &&
-             m_Domain     == other.m_Domain &&
-             m_AuthID     == other.m_AuthID &&
-             m_Password   == other.m_Password &&
-             m_TimeToLive == other.m_TimeToLive &&
-             m_Proxy      == other.m_Proxy &&
-             m_MWI        == other.m_MWI &&
-             m_Presence   == other.m_Presence;
-    }
+    RegistrationInfo();
 
+    bool operator==(const RegistrationInfo & other) const;
+
+    bool Read(wxConfigBase & config);
+    void Write(wxConfigBase & config);
+    bool Start(SIPEndPoint & sipEP);
+    bool Stop(SIPEndPoint & sipEP);
+
+    Types     m_Type;
     bool      m_Active;
     PwxString m_User;
     PwxString m_Domain;
+    PwxString m_Contact;
     PwxString m_AuthID;
     PwxString m_Password;
     int       m_TimeToLive;
     PwxString m_Proxy;
-    bool      m_MWI;
-    bool      m_Presence;
+    PString   m_aor;
 };
 
-typedef list<RegistrarInfo> RegistrarList;
+typedef list<RegistrationInfo> RegistrationList;
+
+
+class RegistrationDialog : public wxDialog
+{
+  public:
+    RegistrationDialog(wxDialog *parent, const RegistrationInfo * info);
+
+    const RegistrationInfo & GetInfo() const { return m_info; }
+
+  private:
+    DECLARE_EVENT_TABLE()
+
+    void Changed(wxCommandEvent & event);
+
+    wxButton       * m_ok;
+    wxTextCtrl     * m_user;
+    wxTextCtrl     * m_domain;
+    RegistrationInfo m_info;
+};
 
 
 class OptionsDialog : public wxDialog
@@ -437,9 +583,9 @@ class OptionsDialog : public wxDialog
     PwxString m_RingSoundDeviceName;
     PwxString m_RingSoundFileName;
     bool      m_AutoAnswer;
-#if OPAL_IVR
-    PwxString m_IVRScript;
-#endif
+    PwxString m_VendorName;
+    PwxString m_ProductName;
+    PwxString m_ProductVersion;
 
     void BrowseSoundFile(wxCommandEvent & event);
     void PlaySoundFile(wxCommandEvent & event);
@@ -454,6 +600,7 @@ class OptionsDialog : public wxDialog
     int             m_RTPPortBase;
     int             m_RTPPortMax;
     int             m_RTPTOS;
+    int             m_MaxRtpPayloadSize;
     wxRadioButton * m_NoNATUsedRadio;
     wxRadioButton * m_NATRouterRadio;
     wxRadioButton * m_STUNServerRadio;
@@ -510,14 +657,23 @@ class OptionsDialog : public wxDialog
     PwxString m_VideoMinFrameSize;
     PwxString m_VideoMaxFrameSize;
 
+    void TestVideoCapture(wxCommandEvent & event);
+
     ////////////////////////////////////////
     // Fax fields
     PwxString m_FaxStationIdentifier;
     PwxString m_FaxReceiveDirectory;
     PwxString m_FaxSpanDSP;
+    int       m_FaxSyncMode;
     void BrowseFaxDirectory(wxCommandEvent & event);
     void BrowseFaxSpanDSP(wxCommandEvent & event);
 
+    ////////////////////////////////////////
+    // IVR fields
+#if OPAL_IVR
+    PwxString m_IVRScript;
+#endif
+
     ////////////////////////////////////////
     // Codec fields
     wxButton     * m_AddCodec;
@@ -577,29 +733,25 @@ class OptionsDialog : public wxDialog
     PwxString m_SIPProxy;
     PwxString m_SIPProxyUsername;
     PwxString m_SIPProxyPassword;
+    int       m_LineAppearanceCode;
 
-    wxListCtrl * m_Registrars;
-    int          m_SelectedRegistrar;
-    wxButton   * m_AddRegistrar;
-    wxButton   * m_ChangeRegistrar;
-    wxButton   * m_RemoveRegistrar;
-    wxTextCtrl * m_RegistrarUser;
-    wxTextCtrl * m_RegistrarDomain;
-    wxTextCtrl * m_RegistrarAuthID;
-    wxTextCtrl * m_RegistrarPassword;
-    wxSpinCtrl * m_RegistrarTimeToLive;
-    wxCheckBox * m_RegistrarActive;
-    wxCheckBox * m_SubscribeMWI;
-    wxCheckBox * m_SubscribePresence;
+    wxListCtrl * m_Registrations;
+    int          m_SelectedRegistration;
+    wxButton   * m_ChangeRegistration;
+    wxButton   * m_RemoveRegistration;
+    wxButton   * m_MoveUpRegistration;
+    wxButton   * m_MoveDownRegistration;
 
-    void FieldsToRegistrar(RegistrarInfo & info);
-    void RegistrarToList(bool overwrite, RegistrarInfo * registrar, int position);
-    void AddRegistrar(wxCommandEvent & event);
-    void ChangeRegistrar(wxCommandEvent & event);
-    void RemoveRegistrar(wxCommandEvent & event);
-    void SelectedRegistrar(wxListEvent & event);
-    void DeselectedRegistrar(wxListEvent & event);
-    void ChangedRegistrarInfo(wxCommandEvent & event);
+    void RegistrationToList(bool create, RegistrationInfo * registration, int position);
+    void AddRegistration(wxCommandEvent & event);
+    void ChangeRegistration(wxCommandEvent & event);
+    void RemoveRegistration(wxCommandEvent & event);
+    void MoveUpRegistration(wxCommandEvent & event);
+    void MoveDownRegistration(wxCommandEvent & event);
+    void MoveRegistration(int delta);
+    void SelectedRegistration(wxListEvent & event);
+    void DeselectedRegistration(wxListEvent & event);
+    void ActivateRegistration(wxListEvent & event);
 
     ////////////////////////////////////////
     // Routing fields
@@ -650,22 +802,21 @@ class MyMedia
 public:
   MyMedia();
   MyMedia(
-    const wxChar * source,
+    const char * source,
     const PString & format
   );
 
   bool operator<(const MyMedia & other) { return preferenceOrder < other.preferenceOrder; }
 
-  const wxChar    * sourceProtocol;
+  PString         sourceProtocol;
   OpalMediaFormat mediaFormat;
-  const wxChar    * validProtocols;
+  const wxChar  * validProtocols;
   int             preferenceOrder;
   bool            dirty;
 };
 
 typedef std::list<MyMedia> MyMediaList;
 
-
 class MyManager : public wxFrame, public OpalManager
 {
   public:
@@ -677,7 +828,7 @@ class MyManager : public wxFrame, public OpalManager
     bool HasSpeedDialName(const wxString & name) const;
     bool HasSpeedDialNumber(const wxString & number, const wxString & ignore) const;
 
-    void MakeCall(const PwxString & address, const PwxString & local = wxEmptyString);
+    void MakeCall(const PwxString & address, const PwxString & local = wxEmptyString, OpalConnection::StringOptions * options = NULL);
     void AnswerCall();
     void RejectCall();
     void HangUpCall();
@@ -750,6 +901,8 @@ class MyManager : public wxFrame, public OpalManager
     void OnAdjustMenus(wxMenuEvent& event);
     void OnStateChange(wxCommandEvent & event);
     void OnStreamsChanged(wxCommandEvent &);
+    void OnRxMessage(wxCommandEvent &);
+    void OnPresence(wxCommandEvent &);
 
     void OnMenuQuit(wxCommandEvent& event);
     void OnMenuAbout(wxCommandEvent& event);
@@ -760,6 +913,7 @@ class MyManager : public wxFrame, public OpalManager
     void OnCallSpeedDialHandset(wxCommandEvent& event);
     void OnSendFax(wxCommandEvent& event);
     void OnSendFaxSpeedDial(wxCommandEvent& event);
+    void OnStartIM(wxCommandEvent& event);
     void OnMenuAnswer(wxCommandEvent& event);
     void OnMenuHangUp(wxCommandEvent& event);
     void OnNewSpeedDial(wxCommandEvent& event);
@@ -778,6 +932,8 @@ class MyManager : public wxFrame, public OpalManager
     void OnTransfer(wxCommandEvent& event);
     void OnStartRecording(wxCommandEvent& event);
     void OnStopRecording(wxCommandEvent& event);
+    void OnAudioDevicePair(wxCommandEvent& event);
+    void OnAudioDevicePreset(wxCommandEvent& event);
     void OnNewCodec(wxCommandEvent& event);
     void OnStartVideo(wxCommandEvent& event);
     void OnStopVideo(wxCommandEvent& event);
@@ -788,6 +944,8 @@ class MyManager : public wxFrame, public OpalManager
     void OnSpeedDialColumnResize(wxListEvent& event);
     void OnRightClick(wxListEvent& event);
 
+    void OnMyPresence(wxCommandEvent& event);
+
     bool CanDoFax() const;
 
     enum SpeedDialViews {
@@ -808,8 +966,10 @@ class MyManager : public wxFrame, public OpalManager
 
     enum {
       e_NameColumn,
+      e_StatusColumn,
       e_NumberColumn,
       e_AddressColumn,
+      e_StateUrlColumn,
       e_DescriptionColumn,
       e_NumColumns
     };
@@ -817,17 +977,16 @@ class MyManager : public wxFrame, public OpalManager
     // Controls on main frame
     enum {
       SplitterID = 100,
+      TabsID,
       SpeedDialsID
     };
 
     wxSplitterWindow * m_splitter;
+    wxNotebook       * m_tabs;
     wxTextCtrl       * m_logWindow;
     wxListCtrl       * m_speedDials;
     wxImageList      * m_imageListNormal;
     wxImageList      * m_imageListSmall;
-    AnswerPanel      * m_answerPanel;
-    CallingPanel     * m_callingPanel;
-    InCallPanel      * m_inCallPanel;
     wxDataFormat       m_ClipboardFormat;
 
     MyPCSSEndPoint   * pcssEP;
@@ -839,7 +998,7 @@ class MyManager : public wxFrame, public OpalManager
     PwxString m_STUNServer;
     void SetNATHandling();
 
-    PStringArray m_LocalInterfaces;
+    vector<PwxString> m_LocalInterfaces;
     void StartAllListeners();
 
 #if OPAL_H323
@@ -851,11 +1010,24 @@ class MyManager : public wxFrame, public OpalManager
     bool StartGatekeeper();
 
 #if OPAL_SIP
-    MySIPEndPoint * sipEP;
-    bool            m_SIPProxyUsed;
-    RegistrarList   m_registrars;
-    void StartRegistrars();
-    void StopRegistrars();
+    friend class MySIPEndPoint;
+    MySIPEndPoint  * sipEP;
+    bool             m_SIPProxyUsed;
+    RegistrationList m_registrations;
+
+    void StartRegistrations();
+    void ReplaceRegistrations(const RegistrationList & newRegistrations);
+
+    typedef std::map<std::string, IMDialog *> ConversationMapType;
+    PMutex conversationMapMutex;
+
+    void OnMessageReceived(MessageInfo * info);
+    IMDialog * GetOrCreateConversation(const MessageInfo & messageInfo);
+
+    ConversationMapType conversationMap;
+
+    void OnPresenceInfoReceived(const SIPPresenceInfo & info);
+
 #endif
 
 #if OPAL_IVR
@@ -863,7 +1035,7 @@ class MyManager : public wxFrame, public OpalManager
 #endif
 
 #if OPAL_FAX
-    OpalT38EndPoint  * m_faxEP;
+    OpalFaxEndPoint  * m_faxEP;
 #endif
 
     bool      m_autoAnswer;
@@ -881,7 +1053,7 @@ class MyManager : public wxFrame, public OpalManager
     bool AdjustFrameSize();
 
     MyMediaList m_mediaInfo;
-    void InitMediaInfo(const wxChar * source, const OpalMediaFormatList & formats);
+    void InitMediaInfo(const char * source, const OpalMediaFormatList & formats);
     void ApplyMediaInfo();
 
 #if PTRACING
@@ -895,6 +1067,7 @@ class MyManager : public wxFrame, public OpalManager
     PTimer        m_RingSoundTimer;
     PDECLARE_NOTIFIER(PTimer, MyManager, OnRingSoundAgain);
     void StopRingSound();
+    void UpdateAudioDevices();
 
     enum CallState {
       IdleState,
@@ -907,7 +1080,7 @@ class MyManager : public wxFrame, public OpalManager
     friend ostream & operator<<(ostream & strm, CallState state);
     void SetState(
       CallState newState,
-      const char * token = NULL
+      const PString & token
     );
 
     PString            m_incomingToken;
@@ -919,6 +1092,10 @@ class MyManager : public wxFrame, public OpalManager
     bool RemoveCallOnHold(
       const PString & token
     );
+    void OnHoldChanged(
+      const PString & token,
+      bool onHold
+    );
 
     struct CallsOnHold {
       CallsOnHold() { }
@@ -930,13 +1107,13 @@ class MyManager : public wxFrame, public OpalManager
     };
     list<CallsOnHold>    m_callsOnHold;
 
-    PFilePath m_lastRecordFile;
-    PFilePath m_faxReceiveDirectory;
+    PwxString m_lastRecordFile;
 
     DECLARE_EVENT_TABLE()
 
   friend class OptionsDialog;
   friend class InCallPanel;
+  friend class IMDialog;
 };
 
 
