@@ -26,9 +26,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 21004 $
+ * $Revision: 21806 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-12-16 02:52:00 +0000 (Tue, 16 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -89,10 +89,6 @@ H323EndPoint::H323EndPoint(OpalManager & manager)
 
   localAliasNames.AppendString(defaultLocalPartyName);
 
-  autoStartReceiveFax = autoStartTransmitFax = PFalse;
-  
-  isH224Enabled = PFalse;
-
   m_bH245Disabled = PFalse;
   autoCallForward = PTrue;
   disableFastStart = PFalse;
@@ -603,14 +599,18 @@ PBoolean H323EndPoint::InternalMakeCall(OpalCall & call,
     return PFalse;
   }
 
-  // Restriction: the call must be made on the same transport as the one
+  // Restriction: the call must be made on the same local interface as the one
   // that the gatekeeper is using.
   H323Transport * transport;
   if (gatekeeper != NULL)
     transport = gatekeeper->GetTransport().GetLocalAddress().CreateTransport(
                                           *this, OpalTransportAddress::Streamed);
-  else
+  else if (stringOptions == NULL || !stringOptions->Contains("interface"))
     transport = address.CreateTransport(*this, OpalTransportAddress::NoBinding);
+  else {
+    OpalTransportAddress localInterface = (*stringOptions)["interface"];
+    transport = localInterface.CreateTransport(*this, OpalTransportAddress::HostOnly);
+  }
 
   if (transport == NULL) {
     PTRACE(1, "H323\tInvalid transport in \"" << remoteParty << '"');
@@ -755,8 +755,8 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
         PIPSocketAddressAndPortVector addresses;
         if (PDNS::LookupSRV(hostname, "_h323._tcp", url.GetPort(), addresses) && !addresses.empty()) {
           // Only use first entry
-          url.SetHostName(addresses[0].address.AsString());
-          url.SetPort(addresses[0].port);
+          url.SetHostName(addresses[0].GetAddress().AsString());
+          url.SetPort(addresses[0].GetPort());
         }
       }
     }
@@ -1165,6 +1165,13 @@ H323ServiceControlSession * H323EndPoint::CreateServiceControlSession(const H225
 }
 
 
+void H323EndPoint::SetDefaultLocalPartyName(const PString & name)
+{
+  SetLocalUserName(name);
+  OpalEndPoint::SetDefaultLocalPartyName(name);
+}
+
+
 void H323EndPoint::SetLocalUserName(const PString & name)
 {
   PAssert(!name, "Must have non-empty string in AliasAddress!");
