@@ -23,97 +23,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: h323trans.cxx,v $
- * Revision 2.2  2005/01/16 23:07:34  csoutheren
- * Fixed problem with IPv6 INADDR_ANY
- *
- * Revision 2.1  2004/02/19 10:47:04  rjongbloed
- * Merged OpenH323 version 1.13.1 changes.
- *
- * Revision 1.23  2003/12/11 05:41:00  csoutheren
- * Added storage of H.225 version in endpoint structure
- * Disabled sending RIPs to endpoints that cannot handle them
- *
- * Revision 1.22  2003/10/27 01:55:33  csoutheren
- * Guarded against possible NULL interface
- *
- * Revision 1.21  2003/04/30 07:51:08  robertj
- * Redesigned the alternate credentials in ARQ system as old implementation
- *   was fraught with concurrency issues, most importantly it can cause false
- *   detection of replay attacks taking out an endpoint completely.
- *
- * Revision 1.20  2003/04/10 09:45:14  robertj
- * Added associated transport to new GetInterfaceAddresses() function so
- *   interfaces can be ordered according to active transport links. Improves
- *   interoperability.
- * Replaced old listener GetTransportPDU() with GetInterfaceAddresses()
- *   and H323SetTransportAddresses() functions.
- *
- * Revision 1.19  2003/04/10 01:00:41  craigs
- * Added functions to access to lists of interfaces
- *
- * Revision 1.18  2003/04/09 03:50:58  craigs
- * Fixed problem with peer elements
- *
- * Revision 1.17  2003/04/09 03:08:10  robertj
- * Fixed race condition in shutting down transactor (pure virtual call)
- *
- * Revision 1.16  2003/04/02 06:08:31  robertj
- * Fixed some trace log module names
- *
- * Revision 1.15  2003/04/01 05:59:33  robertj
- * Fixed H.501 transaction code setting members for m_common PDU part.
- *
- * Revision 1.14  2003/04/01 04:47:55  robertj
- * Abstracted H.225 RAS transaction processing (RIP and secondary thread) in
- *   server environment for use by H.501 peer elements.
- *
- * Revision 1.13  2003/03/26 00:46:29  robertj
- * Had another go at making H323Transactor being able to be created
- *   without having a listener running.
- *
- * Revision 1.12  2003/03/25 12:01:19  craigs
- * Fixed SEGV when no interface specified for peer element
- *
- * Revision 1.11  2003/03/25 08:12:54  robertj
- * Added ability to create a transactor without starting listener.
- *
- * Revision 1.10  2003/03/25 04:56:21  robertj
- * Fixed issues to do with multiple inheritence in transaction reply cache.
- *
- * Revision 1.9  2003/03/25 02:32:15  robertj
- * Additional tracing.
- * Fixed bug with incorrect authenticators being used in requests.
- *
- * Revision 1.8  2003/03/25 01:41:02  craigs
- * Still more signficant H.501 updates
- *
- * Revision 1.7  2003/03/24 07:16:13  robertj
- * Fixed incorrect cast to non const for setting authenticators.
- *
- * Revision 1.6  2003/03/21 06:27:55  robertj
- * Added setting of remote port in UDP transport constructor.
- * Removed odd trace output on timeout..
- *
- * Revision 1.5  2003/03/20 01:51:12  robertj
- * More abstraction of H.225 RAS and H.501 protocols transaction handling.
- *
- * Revision 1.4  2003/03/01 00:22:10  craigs
- * New PeerElement implementation
- *
- * Revision 1.3  2003/02/25 06:48:19  robertj
- * More work on PDU transaction abstraction.
- *
- * Revision 1.2  2003/02/21 07:23:18  robertj
- * Fixed up some comments
- *
- * Revision 1.1  2003/02/21 05:27:06  craigs
- * Initial version
- *
+ * $Revision: 22169 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-09 02:18:57 +0000 (Mon, 09 Mar 2009) $
  */
 
 #include <ptlib.h>
 
+#include <opal/buildopts.h>
+#if OPAL_H323
+
 #ifdef __GNUC__
 #pragma implementation "h323trans.h"
 #endif
@@ -132,6 +51,20 @@ static PTimeInterval ResponseRetirementAge(0, 30); // Seconds
 #define new PNEW
 
 
+static struct AuthenticatorString {
+  int code;
+  const char * desc;
+} authenticatorStrings[] = {
+  { H235Authenticator::e_OK,          "Security parameters and Msg are ok, no security attacks" },
+  { H235Authenticator::e_Absent,      "Security parameters are expected but absent" },
+  { H235Authenticator::e_Error,       "Security parameters are present but incorrect" },
+  { H235Authenticator::e_InvalidTime, "Security parameters indicate peer has bad real time clock" },
+  { H235Authenticator::e_BadPassword, "Security parameters indicate bad password in token" },
+  { H235Authenticator::e_ReplyAttack, "Security parameters indicate an attack was made" },
+  { H235Authenticator::e_Disabled,    "Security is disabled by local system" },
+  { -1, NULL }
+};
+
 /////////////////////////////////////////////////////////////////////////////////
 
 H323TransactionPDU::H323TransactionPDU()
@@ -152,49 +85,49 @@ ostream & operator<<(ostream & strm, const H323TransactionPDU & pdu)
 }
 
 
-BOOL H323TransactionPDU::Read(H323Transport & transport)
+PBoolean H323TransactionPDU::Read(H323Transport & transport)
 {
   if (!transport.ReadPDU(rawPDU)) {
     PTRACE(1, GetProtocolName() << "\tRead error ("
            << transport.GetErrorNumber(PChannel::LastReadError)
            << "): " << transport.GetErrorText(PChannel::LastReadError));
-    return FALSE;
+    return PFalse;
   }
 
   rawPDU.ResetDecoder();
-  BOOL ok = GetPDU().Decode(rawPDU);
+  PBoolean ok = GetPDU().Decode(rawPDU);
   if (!ok) {
     PTRACE(1, GetProtocolName() << "\tRead error: PER decode failure:\n  "
            << setprecision(2) << rawPDU << "\n "  << setprecision(2) << *this);
     GetChoice().SetTag(UINT_MAX);
-    return TRUE;
+    return PTrue;
   }
 
-  H323TraceDumpPDU(GetProtocolName(), FALSE, rawPDU, GetPDU(), GetChoice(), GetSequenceNumber());
+  H323TraceDumpPDU(GetProtocolName(), PFalse, rawPDU, GetPDU(), GetChoice(), GetSequenceNumber());
 
-  return TRUE;
+  return PTrue;
 }
 
 
-BOOL H323TransactionPDU::Write(H323Transport & transport)
+PBoolean H323TransactionPDU::Write(H323Transport & transport)
 {
   PPER_Stream strm;
   GetPDU().Encode(strm);
   strm.CompleteEncoding();
 
   // Finalise the security if present
-  for (PINDEX i = 0; i < authenticators.GetSize(); i++)
-    authenticators[i].Finalise(strm);
+  for (H235Authenticators::iterator iterAuth = authenticators.begin(); iterAuth != authenticators.end(); ++iterAuth)
+    iterAuth->Finalise(strm);
 
-  H323TraceDumpPDU("Trans", TRUE, strm, GetPDU(), GetChoice(), GetSequenceNumber());
+  H323TraceDumpPDU("Trans", PTrue, strm, GetPDU(), GetChoice(), GetSequenceNumber());
 
   if (transport.WritePDU(strm))
-    return TRUE;
+    return PTrue;
 
   PTRACE(1, GetProtocolName() << "\tWrite PDU failed ("
          << transport.GetErrorNumber(PChannel::LastWriteError)
          << "): " << transport.GetErrorText(PChannel::LastWriteError));
-  return FALSE;
+  return PFalse;
 }
 
 
@@ -211,7 +144,7 @@ H323Transactor::H323Transactor(H323EndPoint & ep,
   if (trans != NULL)
     transport = trans;
   else
-    transport = new H323TransportUDP(ep, INADDR_ANY, local_port, remote_port);
+    transport = new H323TransportUDP(ep, PIPSocket::GetDefaultIpAny(), local_port);
 
   Construct();
 }
@@ -230,7 +163,7 @@ H323Transactor::H323Transactor(H323EndPoint & ep,
   else {
     PIPSocket::Address addr;
     PAssert(iface.GetIpAndPort(addr, local_port), "Cannot parse address");
-    transport = new H323TransportUDP(ep, addr, local_port, remote_port);
+    transport = new H323TransportUDP(ep, addr, local_port);
   }
 
   Construct();
@@ -240,7 +173,7 @@ H323Transactor::H323Transactor(H323EndPoint & ep,
 void H323Transactor::Construct()
 {
   nextSequenceNumber = PRandom::Number()%65536;
-  checkResponseCryptoTokens = TRUE;
+  checkResponseCryptoTokens = PTrue;
   lastRequest = NULL;
 
   requests.DisallowDeleteObjects();
@@ -273,20 +206,20 @@ void H323Transactor::PrintOn(ostream & strm) const
 }
 
 
-BOOL H323Transactor::SetTransport(const H323TransportAddress & iface)
+PBoolean H323Transactor::SetTransport(const H323TransportAddress & iface)
 {
   PWaitAndSignal mutex(pduWriteMutex);
 
   if (transport != NULL && transport->GetLocalAddress().IsEquivalent(iface)) {
     PTRACE(2, "Trans\tAlready have listener for " << iface);
-    return TRUE;
+    return PTrue;
   }
 
   PIPSocket::Address addr;
   WORD port = defaultLocalPort;
   if (!iface.GetIpAndPort(addr, port)) {
-    PTRACE(2, "Trans\tCannot create listener for " << iface);
-    return FALSE;
+    PTRACE(1, "Trans\tCannot create listener for " << iface);
+    return PFalse;
   }
 
   if (transport != NULL) {
@@ -294,32 +227,28 @@ BOOL H323Transactor::SetTransport(const H323TransportAddress & iface)
     delete transport;
   }
 
-    transport = new H323TransportUDP(endpoint, addr, port, defaultRemotePort);
+  transport = new H323TransportUDP(endpoint, addr, port);
   transport->SetPromiscuous(H323Transport::AcceptFromAny);
   return StartChannel();;
 }
 
-H323TransportAddressArray H323Transactor::GetInterfaceAddresses(BOOL excludeLocalHost,
-                                                                H323Transport * associatedTransport)
+
+H323TransportAddressArray H323Transactor::GetInterfaceAddresses(bool excludeLocalHost)
 {
   if (transport == NULL)
     return H323TransportAddressArray();
   else
-    return OpalGetInterfaceAddresses(transport->GetLocalAddress(),
-                                     excludeLocalHost,
-                                     associatedTransport);
+    return endpoint.GetInterfaceAddresses(excludeLocalHost, transport);
 }
 
-BOOL H323Transactor::StartChannel()
+
+PBoolean H323Transactor::StartChannel()
 {
   if (transport == NULL)
-    return FALSE;
+    return PFalse;
 
-  transport->AttachThread(PThread::Create(PCREATE_NOTIFIER(HandleTransactions), 0,
-                                          PThread::NoAutoDeleteThread,
-                                          PThread::NormalPriority,
-                                          "Transactor:%x"));
-  return TRUE;
+  transport->AttachThread(PThread::Create(PCREATE_NOTIFIER(HandleTransactions), "Transactor"));
+  return PTrue;
 }
 
 
@@ -338,13 +267,13 @@ void H323Transactor::HandleTransactions(PThread &, INT)
   if (PAssertNULL(transport) == NULL)
     return;
 
-  PTRACE(2, "Trans\tStarting listener thread on " << *transport);
+  PTRACE(3, "Trans\tStarting listener thread on " << *transport);
 
   transport->SetReadTimeout(PMaxTimeInterval);
 
   PINDEX consecutiveErrors = 0;
 
-  BOOL ok = TRUE;
+  PBoolean ok = PTrue;
   while (ok) {
     PTRACE(5, "Trans\tReading PDU");
     H323TransactionPDU * response = CreateTransactionPDU();
@@ -364,7 +293,7 @@ void H323Transactor::HandleTransactions(PThread &, INT)
           // Do NotOpen case
 
         case PChannel::NotOpen :
-          ok = FALSE;
+          ok = PFalse;
           break;
 
         default :
@@ -377,7 +306,7 @@ void H323Transactor::HandleTransactions(PThread &, INT)
             default:
               PTRACE(1, "Trans\tRead error: " << transport->GetErrorText(PChannel::LastReadError));
               if (++consecutiveErrors > 10)
-                ok = FALSE;
+                ok = PFalse;
           }
       }
     }
@@ -386,17 +315,17 @@ void H323Transactor::HandleTransactions(PThread &, INT)
     AgeResponses();
   }
 
-  PTRACE(2, "Trans\tEnded listener thread on " << *transport);
+  PTRACE(3, "Trans\tEnded listener thread on " << *transport);
 }
 
 
-BOOL H323Transactor::SetUpCallSignalAddresses(H225_ArrayOf_TransportAddress & addresses)
+PBoolean H323Transactor::SetUpCallSignalAddresses(H225_ArrayOf_TransportAddress & addresses)
 {
   if (PAssertNULL(transport) == NULL)
-    return FALSE;
+    return PFalse;
 
   H323SetTransportAddresses(*transport,
-                            endpoint.GetInterfaceAddresses(FALSE, transport),
+                            endpoint.GetInterfaceAddresses(PFalse, transport),
                             addresses);
 
   return addresses.GetSize() > 0;
@@ -429,10 +358,10 @@ void H323Transactor::AgeResponses()
 }
 
 
-BOOL H323Transactor::SendCachedResponse(const H323TransactionPDU & pdu)
+PBoolean H323Transactor::SendCachedResponse(const H323TransactionPDU & pdu)
 {
   if (PAssertNULL(transport) == NULL)
-    return FALSE;
+    return PFalse;
 
   Response key(transport->GetLastReceivedAddress(), pdu.GetSequenceNumber());
 
@@ -443,14 +372,14 @@ BOOL H323Transactor::SendCachedResponse(const H323TransactionPDU & pdu)
     return responses[idx].SendCachedResponse(*transport);
 
   responses.Append(new Response(key));
-  return FALSE;
+  return PFalse;
 }
 
 
-BOOL H323Transactor::WritePDU(H323TransactionPDU & pdu)
+PBoolean H323Transactor::WritePDU(H323TransactionPDU & pdu)
 {
   if (PAssertNULL(transport) == NULL)
-    return FALSE;
+    return PFalse;
 
   OnSendingPDU(pdu.GetPDU());
 
@@ -465,12 +394,12 @@ BOOL H323Transactor::WritePDU(H323TransactionPDU & pdu)
 }
 
 
-BOOL H323Transactor::WriteTo(H323TransactionPDU & pdu,
+PBoolean H323Transactor::WriteTo(H323TransactionPDU & pdu,
                              const H323TransportAddressArray & addresses,
-                             BOOL callback)
+                             PBoolean callback)
 {
   if (PAssertNULL(transport) == NULL)
-    return FALSE;
+    return PFalse;
 
   if (addresses.IsEmpty()) {
     if (callback)
@@ -483,9 +412,9 @@ BOOL H323Transactor::WriteTo(H323TransactionPDU & pdu,
 
   H323TransportAddress oldAddress = transport->GetRemoteAddress();
 
-  BOOL ok = FALSE;
+  PBoolean ok = PFalse;
   for (PINDEX i = 0; i < addresses.GetSize(); i++) {
-    if (transport->ConnectTo(addresses[i])) {
+    if (transport->SetRemoteAddress(addresses[i])) {
       PTRACE(3, "Trans\tWrite address set to " << addresses[i]);
       if (callback)
         ok = WritePDU(pdu);
@@ -494,7 +423,7 @@ BOOL H323Transactor::WriteTo(H323TransactionPDU & pdu,
     }
   }
 
-  transport->ConnectTo(oldAddress);
+  transport->SetRemoteAddress(oldAddress);
 
   pduWriteMutex.Signal();
 
@@ -502,7 +431,7 @@ BOOL H323Transactor::WriteTo(H323TransactionPDU & pdu,
 }
 
 
-BOOL H323Transactor::MakeRequest(Request & request)
+PBoolean H323Transactor::MakeRequest(Request & request)
 {
   PTRACE(3, "Trans\tMaking request: " << request.requestPDU.GetChoice().GetTagName());
 
@@ -512,7 +441,7 @@ BOOL H323Transactor::MakeRequest(Request & request)
   requests.SetAt(request.sequenceNumber, &request);
   requestsMutex.Signal();
 
-  BOOL ok = request.Poll(*this);
+  PBoolean ok = request.Poll(*this);
 
   requestsMutex.Wait();
   requests.SetAt(request.sequenceNumber, NULL);
@@ -522,24 +451,24 @@ BOOL H323Transactor::MakeRequest(Request & request)
 }
 
 
-BOOL H323Transactor::CheckForResponse(unsigned reqTag, unsigned seqNum, const PASN_Choice * reason)
+PBoolean H323Transactor::CheckForResponse(unsigned reqTag, unsigned seqNum, const PASN_Choice * reason)
 {
   requestsMutex.Wait();
   lastRequest = requests.GetAt(seqNum);
   requestsMutex.Signal();
 
   if (lastRequest == NULL) {
-    PTRACE(3, "Trans\tTimed out or received sequence number (" << seqNum << ") for PDU we never requested");
-    return FALSE;
+    PTRACE(2, "Trans\tTimed out or received sequence number (" << seqNum << ") for PDU we never requested");
+    return PFalse;
   }
 
   lastRequest->responseMutex.Wait();
   lastRequest->CheckResponse(reqTag, reason);
-  return TRUE;
+  return PTrue;
 }
 
 
-BOOL H323Transactor::HandleRequestInProgress(const H323TransactionPDU & pdu,
+PBoolean H323Transactor::HandleRequestInProgress(const H323TransactionPDU & pdu,
                                              unsigned delay)
 {
   unsigned seqNum = pdu.GetSequenceNumber();
@@ -549,27 +478,27 @@ BOOL H323Transactor::HandleRequestInProgress(const H323TransactionPDU & pdu,
   requestsMutex.Signal();
 
   if (lastRequest == NULL) {
-    PTRACE(3, "Trans\tTimed out or received sequence number (" << seqNum << ") for PDU we never requested");
-    return FALSE;
+    PTRACE(2, "Trans\tTimed out or received sequence number (" << seqNum << ") for PDU we never requested");
+    return PFalse;
   }
 
   lastRequest->responseMutex.Wait();
 
   PTRACE(3, "Trans\tReceived RIP on sequence number " << seqNum);
   lastRequest->OnReceiveRIP(delay);
-  return TRUE;
+  return PTrue;
 }
 
 
-BOOL H323Transactor::CheckCryptoTokens(const H323TransactionPDU & pdu,
+PBoolean H323Transactor::CheckCryptoTokens(const H323TransactionPDU & pdu,
                                        const PASN_Array & clearTokens,
                                        unsigned clearOptionalField,
                                        const PASN_Array & cryptoTokens,
                                        unsigned cryptoOptionalField)
 {
-  // If cypto token checking disabled, just return TRUE.
+  // If cypto token checking disabled, just return PTrue.
   if (!GetCheckResponseCryptoTokens())
-    return TRUE;
+    return PTrue;
 
   if (lastRequest != NULL && pdu.GetAuthenticators().IsEmpty()) {
     ((H323TransactionPDU &)pdu).SetAuthenticators(lastRequest->requestPDU.GetAuthenticators());
@@ -579,7 +508,7 @@ BOOL H323Transactor::CheckCryptoTokens(const H323TransactionPDU & pdu,
 
   if (pdu.Validate(clearTokens, clearOptionalField,
                    cryptoTokens, cryptoOptionalField) == H235Authenticator::e_OK)
-    return TRUE;
+    return PTrue;
 
   /* Note that a crypto tokens error is flagged to the requestor in the
      responseResult field but the other thread is NOT signalled. This is so
@@ -593,7 +522,7 @@ BOOL H323Transactor::CheckCryptoTokens(const H323TransactionPDU & pdu,
     lastRequest = NULL;
   }
 
-  return FALSE;
+  return PFalse;
 }
 
 
@@ -618,21 +547,27 @@ H323Transactor::Request::Request(unsigned seqNum,
 }
 
 
-BOOL H323Transactor::Request::Poll(H323Transactor & rasChannel)
+PBoolean H323Transactor::Request::Poll(H323Transactor & rasChannel, unsigned numRetries, PTimeInterval timeout)
 {
   H323EndPoint & endpoint = rasChannel.GetEndPoint();
 
   responseResult = AwaitingResponse;
+  
+  if (numRetries == 0)
+    numRetries = endpoint.GetRasRequestRetries();
+  
+  if (timeout == 0)
+    timeout = endpoint.GetRasRequestTimeout();
 
-  for (unsigned retry = 1; retry <= endpoint.GetRasRequestRetries(); retry++) {
+  for (unsigned retry = 1; retry <= numRetries; retry++) {
     // To avoid race condition with RIP must set timeout before sending the packet
-    whenResponseExpected = PTimer::Tick() + endpoint.GetRasRequestTimeout();
+    whenResponseExpected = PTimer::Tick() + timeout;
 
-    if (!rasChannel.WriteTo(requestPDU, requestAddresses, FALSE))
+    if (!rasChannel.WriteTo(requestPDU, requestAddresses, PFalse))
       break;
 
     PTRACE(3, "Trans\tWaiting on response to seqnum=" << requestPDU.GetSequenceNumber()
-           << " for " << setprecision(1) << endpoint.GetRasRequestTimeout() << " seconds");
+           << " for " << setprecision(1) << timeout << " seconds");
 
     do {
       // Wait for a response
@@ -646,15 +581,16 @@ BOOL H323Transactor::Request::Poll(H323Transactor & rasChannel)
           break;
 
         case ConfirmReceived :
-          return TRUE;
+          return PTrue;
 
         case RejectReceived :
-          return FALSE;
+        case TryAlternate :
+          return PFalse;
 
         case BadCryptoTokens :
-          PTRACE(2, "Trans\tResponse to seqnum=" << requestPDU.GetSequenceNumber()
+          PTRACE(1, "Trans\tResponse to seqnum=" << requestPDU.GetSequenceNumber()
                  << " had invalid crypto tokens.");
-          return FALSE;
+          return PFalse;
 
         default : // RequestInProgress
           responseResult = AwaitingResponse; // Keep waiting
@@ -666,17 +602,17 @@ BOOL H323Transactor::Request::Poll(H323Transactor & rasChannel)
     } while (responseResult == AwaitingResponse);
 
     PTRACE(1, "Trans\tTimeout on request seqnum=" << requestPDU.GetSequenceNumber()
-           << ", try #" << retry << " of " << endpoint.GetRasRequestRetries());
+           << ", try #" << retry << " of " << numRetries);
   }
 
-  return FALSE;
+  return PFalse;
 }
 
 
 void H323Transactor::Request::CheckResponse(unsigned reqTag, const PASN_Choice * reason)
 {
   if (requestPDU.GetChoice().GetTag() != reqTag) {
-    PTRACE(3, "Trans\tReceived reply for incorrect PDU tag.");
+    PTRACE(2, "Trans\tReceived reply for incorrect PDU tag.");
     responseResult = RejectReceived;
     rejectReason = UINT_MAX;
     return;
@@ -687,7 +623,7 @@ void H323Transactor::Request::CheckResponse(unsigned reqTag, const PASN_Choice *
     return;
   }
 
-  PTRACE(1, "Trans\t" << requestPDU.GetChoice().GetTagName()
+  PTRACE(2, "Trans\t" << requestPDU.GetChoice().GetTagName()
          << " rejected: " << reason->GetTagName());
   responseResult = RejectReceived;
   rejectReason = reason->GetTag();
@@ -761,7 +697,7 @@ void H323Transactor::Response::SetPDU(const H323TransactionPDU & pdu)
 }
 
 
-BOOL H323Transactor::Response::SendCachedResponse(H323Transport & transport)
+PBoolean H323Transactor::Response::SendCachedResponse(H323Transport & transport)
 {
   PTRACE(3, "Trans\tSending cached response: " << *this);
 
@@ -776,7 +712,7 @@ BOOL H323Transactor::Response::SendCachedResponse(H323Transport & transport)
   }
 
   lastUsedTime = PTime();
-  return TRUE;
+  return PTrue;
 }
 
 
@@ -793,9 +729,9 @@ H323Transaction::H323Transaction(H323Transactor & trans,
   confirm = conf;
   reject = rej;
   authenticatorResult = H235Authenticator::e_Disabled;
-  fastResponseRequired = TRUE;
-  isBehindNAT = FALSE;
-  canSendRIP  = FALSE;
+  fastResponseRequired = PTrue;
+  isBehindNAT = PFalse;
+  canSendRIP  = PFalse;
 }
 
 
@@ -807,64 +743,64 @@ H323Transaction::~H323Transaction()
 }
 
 
-BOOL H323Transaction::HandlePDU()
+PBoolean H323Transaction::HandlePDU()
 {
   int response = OnHandlePDU();
   switch (response) {
     case Ignore :
-      return FALSE;
+      return PFalse;
 
     case Confirm :
       if (confirm != NULL)
         WritePDU(*confirm);
-      return FALSE;
+      return PFalse;
 
     case Reject :
       if (reject != NULL)
         WritePDU(*reject);
-      return FALSE;
+      return PFalse;
   }
 
   H323TransactionPDU * rip = CreateRIP(request->GetSequenceNumber(), response);
-  BOOL ok = WritePDU(*rip);
+  PBoolean ok = WritePDU(*rip);
   delete rip;
 
   if (!ok)
-    return FALSE;
+    return PFalse;
 
   if (fastResponseRequired) {
-    fastResponseRequired = FALSE;
+    fastResponseRequired = PFalse;
     PThread::Create(PCREATE_NOTIFIER(SlowHandler), 0,
                                      PThread::AutoDeleteThread,
                                      PThread::NormalPriority,
-                                     "Transaction:%x");
+                                     "Transaction");
   }
 
-  return TRUE;
+  return PTrue;
 }
 
 
 void H323Transaction::SlowHandler(PThread &, INT)
 {
-  PTRACE(3, "Trans\tStarted slow PDU handler thread.");
+  PTRACE(4, "Trans\tStarted slow PDU handler thread.");
 
   while (HandlePDU())
     ;
 
   delete this;
 
-  PTRACE(3, "Trans\tEnded slow PDU handler thread.");
+  PTRACE(4, "Trans\tEnded slow PDU handler thread.");
 }
 
 
-BOOL H323Transaction::WritePDU(H323TransactionPDU & pdu)
+PBoolean H323Transaction::WritePDU(H323TransactionPDU & pdu)
 {
   pdu.SetAuthenticators(authenticators);
-  return transactor.WriteTo(pdu, replyAddresses, TRUE);
+  return transactor.WriteTo(pdu, replyAddresses, PTrue);
 }
 
 
-BOOL H323Transaction::CheckCryptoTokens(const H235Authenticators & auth)
+PBoolean H323Transaction::CheckCryptoTokens(const H235Authenticators & auth)
 {
   authenticators = auth;
 
@@ -873,10 +809,18 @@ BOOL H323Transaction::CheckCryptoTokens(const H235Authenticators & auth)
   authenticatorResult = ValidatePDU();
 
   if (authenticatorResult == H235Authenticator::e_OK)
-    return TRUE;
+    return PTrue;
 
-  PTRACE(2, "Trans\t" << GetName() << " rejected, security tokens invalid.");
-  return FALSE;
+  PINDEX i;
+  for (i = 0; (authenticatorStrings[i].code >= 0) && (authenticatorStrings[i].code != authenticatorResult); ++i) 
+    ;
+
+  const char * desc = "Unknown error";
+  if (authenticatorStrings[i].code >= 0)
+    desc = authenticatorStrings[i].desc;
+
+  PTRACE(2, "Trans\t" << GetName() << " rejected - " << desc);
+  return PFalse;
 }
 
 
@@ -893,26 +837,29 @@ H323TransactionServer::~H323TransactionServer()
 }
 
 
-BOOL H323TransactionServer::AddListeners(const H323TransportAddressArray & ifaces)
+PBoolean H323TransactionServer::AddListeners(const H323TransportAddressArray & ifaces)
 {
   if (ifaces.IsEmpty())
-    return AddListener("*");
+    return AddListener("udp$*");
 
   PINDEX i;
 
   mutex.Wait();
-  for (i = 0; i < listeners.GetSize(); i++) {
-    BOOL remove = TRUE;
+  ListenerList::iterator iterListener = listeners.begin();
+  while (iterListener != listeners.end()) {
+    PBoolean remove = PTrue;
     for (PINDEX j = 0; j < ifaces.GetSize(); j++) {
-      if (listeners[i].GetTransport().GetLocalAddress().IsEquivalent(ifaces[j])) {
-        remove = FALSE;
+      if (iterListener->GetTransport().GetLocalAddress().IsEquivalent(ifaces[j], true)) {
+        remove = PFalse;
        break;
       }
     }
     if (remove) {
-      PTRACE(3, "Trans\tRemoving listener " << listeners[i]);
-      listeners.RemoveAt(i--);
+      PTRACE(3, "Trans\tRemoving listener " << *iterListener);
+      listeners.erase(iterListener++);
     }
+    else
+      ++iterListener;
   }
   mutex.Signal();
 
@@ -925,15 +872,15 @@ BOOL H323TransactionServer::AddListeners(const H323TransportAddressArray & iface
 }
 
 
-BOOL H323TransactionServer::AddListener(const H323TransportAddress & interfaceName)
+PBoolean H323TransactionServer::AddListener(const H323TransportAddress & interfaceName)
 {
   PWaitAndSignal wait(mutex);
 
   PINDEX i;
-  for (i = 0; i < listeners.GetSize(); i++) {
-    if (listeners[i].GetTransport().GetLocalAddress().IsEquivalent(interfaceName)) {
+  for (ListenerList::iterator iterListener = listeners.begin(); iterListener != listeners.end(); ++iterListener) {
+    if (iterListener->GetTransport().GetLocalAddress().IsEquivalent(interfaceName, true)) {
       PTRACE(2, "H323\tAlready have listener for " << interfaceName);
-      return TRUE;
+      return PTrue;
     }
   }
 
@@ -942,26 +889,26 @@ BOOL H323TransactionServer::AddListener(const H323TransportAddress & interfaceNa
   if (!interfaceName.GetIpAndPort(addr, port))
     return AddListener(interfaceName.CreateTransport(ownerEndPoint));
 
-  if (addr.IsAny())
+  if (!addr.IsAny())
     return AddListener(new H323TransportUDP(ownerEndPoint, addr, port));
 
   PIPSocket::InterfaceTable interfaces;
   if (!PIPSocket::GetInterfaceTable(interfaces)) {
     PTRACE(1, "Trans\tNo interfaces on system!");
     if (!PIPSocket::GetHostAddress(addr))
-      return FALSE;
+      return PFalse;
     return AddListener(new H323TransportUDP(ownerEndPoint, addr, port));
   }
 
   PTRACE(4, "Trans\tAdding interfaces:\n" << setfill('\n') << interfaces << setfill(' '));
 
-  BOOL atLeastOne = FALSE;
+  PBoolean atLeastOne = PFalse;
 
   for (i = 0; i < interfaces.GetSize(); i++) {
     addr = interfaces[i].GetAddress();
     if (addr != 0) {
-      if (AddListener(new H323TransportUDP(ownerEndPoint, addr, port)))
-        atLeastOne = TRUE;
+      if (AddListener(new H323TransportUDP(ownerEndPoint, addr, port, false, true)))
+        atLeastOne = PTrue;
     }
   }
 
@@ -969,24 +916,24 @@ BOOL H323TransactionServer::AddListener(const H323TransportAddress & interfaceNa
 }
 
 
-BOOL H323TransactionServer::AddListener(H323Transport * transport)
+PBoolean H323TransactionServer::AddListener(H323Transport * transport)
 {
   if (transport == NULL)
-    return FALSE;
+    return PFalse;
 
   if (!transport->IsOpen()) {
     delete transport;
-    return FALSE;
+    return PFalse;
   }
 
   return AddListener(CreateListener(transport));
 }
 
 
-BOOL H323TransactionServer::AddListener(H323Transactor * listener)
+PBoolean H323TransactionServer::AddListener(H323Transactor * listener)
 {
   if (listener == NULL)
-    return FALSE;
+    return PFalse;
 
   PTRACE(3, "Trans\tStarted listener " << *listener);
 
@@ -996,13 +943,13 @@ BOOL H323TransactionServer::AddListener(H323Transactor * listener)
 
   listener->StartChannel();
 
-  return TRUE;
+  return PTrue;
 }
 
 
-BOOL H323TransactionServer::RemoveListener(H323Transactor * listener)
+PBoolean H323TransactionServer::RemoveListener(H323Transactor * listener)
 {
-  BOOL ok = TRUE;
+  PBoolean ok = PTrue;
 
   mutex.Wait();
   if (listener != NULL) {
@@ -1019,4 +966,6 @@ BOOL H323TransactionServer::RemoveListener(H323Transactor * listener)
 }
 
 
+#endif // OPAL_H323
+
 /////////////////////////////////////////////////////////////////////////////////
