@@ -26,236 +26,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: q931.cxx,v $
- * Revision 2.11  2004/11/07 12:45:50  rjongbloed
- * Minor change to parsing of bearer capabilities, thanks Michal Zygmuntowicz
- *
- * Revision 2.10  2004/11/07 12:22:14  rjongbloed
- * Changed generation of call reference to include zero as it is an illegal value
- *   anyway, thanks Dmitriy
- *
- * Revision 2.9  2004/06/06 08:45:11  rjongbloed
- * Fixed incorrect test for Q.931 calls state information element, thanks Stelios Vaiopoulos
- *
- * Revision 2.8  2004/02/19 10:47:04  rjongbloed
- * Merged OpenH323 version 1.13.1 changes.
- *
- * Revision 2.7  2003/01/07 04:39:53  robertj
- * Updated to OpenH323 v1.11.2
- *
- * Revision 2.6  2002/09/04 06:01:49  robertj
- * Updated to OpenH323 v1.9.6
- *
- * Revision 2.5  2002/07/01 04:56:32  robertj
- * Updated to OpenH323 v1.9.1
- *
- * Revision 2.4  2002/01/14 06:35:58  robertj
- * Updated to OpenH323 v1.7.9
- *
- * Revision 2.3  2001/10/05 00:22:14  robertj
- * Updated to PWLib 1.2.0 and OpenH323 1.7.0
- *
- * Revision 2.2  2001/08/21 01:11:31  robertj
- * Update from OpenH323
- *
- * Revision 2.1  2001/08/13 05:10:40  robertj
- * Updates from OpenH323 v1.6.0 release.
- *
- * Revision 2.0  2001/07/27 15:48:25  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
- * Revision 1.59  2003/08/24 23:10:30  rjongbloed
- * Fixed incorrect mask in bearer caps decoding, thanks Greg Adams
- *
- * Revision 1.58  2003/03/18 05:54:18  robertj
- * Added ChannelIdentifier IE support, thanks Eize Slange
- *
- * Revision 1.57  2003/02/12 02:23:01  robertj
- * Fixed printing of cause codes that are impossible as opposed to merely
- *   unlikely or unallocated.
- *
- * Revision 1.56  2003/02/12 00:02:23  robertj
- * Added more Q.931 cause codes.
- * Added ability to trace text version of cause codes and IE codes.
- *
- * Revision 1.55  2002/11/19 06:19:25  robertj
- * Added extra "congested" Q.931 codes.
- *
- * Revision 1.54  2002/08/06 02:27:39  robertj
- * GNU C++ v3 compatibility.
- *
- * Revision 1.53  2002/08/05 10:03:48  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.52  2002/07/25 10:55:44  robertj
- * Changes to allow more granularity in PDU dumps, hex output increasing
- *   with increasing trace level.
- *
- * Revision 1.51  2002/07/05 03:13:38  robertj
- * Added copy constructor for Q.931 so makes duplicate instead of reference to IE's.
- *
- * Revision 1.50  2002/07/03 10:26:05  robertj
- * Fixed bearer capabilities for H,450.1 needing non ITU standard, thanks Florian Winterstein
- *
- * Revision 1.49  2002/05/22 23:12:03  robertj
- * Enhanced the display of Release-Complete cause codes.
- *
- * Revision 1.48  2002/05/07 23:49:35  craigs
- * Changed comment on length of userUserIE field thanks to Paul Long
- *
- * Revision 1.47  2002/05/03 05:38:19  robertj
- * Added Q.931 Keypad IE mechanism for user indications (DTMF).
- *
- * Revision 1.46  2002/04/22 07:32:16  craigs
- * Changed GetProgressIndicator to be const
- *
- * Revision 1.45  2002/04/19 04:49:14  robertj
- * Fixed currect bit shift for CallState IE standard bits.
- *
- * Revision 1.44  2002/04/19 02:16:47  robertj
- * Added CallState IE processing.
- *
- * Revision 1.43  2002/04/18 06:16:32  craigs
- * Extra robustness in handling of strange UserUserIE lengths
- *
- * Revision 1.42  2002/01/07 04:25:21  robertj
- * Added support for Connected-Number Information Element, thanks Hans Verbeek
- *
- * Revision 1.41  2002/01/06 05:28:41  robertj
- * Fixed crash if given bad data in number field, thanks Chih-Wei Huang.
- *
- * Revision 1.40  2001/09/17 02:06:40  robertj
- * Added Redirecting Number IE to Q.931, thanks Frank Derks
- *
- * Revision 1.39  2001/09/13 02:41:21  robertj
- * Fixed call reference generation to use full range and common code, thanks Carlo Kielstra
- *
- * Revision 1.38  2001/08/27 03:45:59  robertj
- * Added automatic setting of bearer capability transfer mode from H.323
- *    capabilities on connection at time of SETUP PDU.
- *
- * Revision 1.37  2001/08/20 06:48:28  robertj
- * Added Q.931 function for setting bearer capabilities, allowing
- *    applications to set the data rate as they require.
- *
- * Revision 1.36  2001/08/07 02:57:09  robertj
- * Fixed incorrect Q.931 bearer capability, thanks Carlo Kielstra.
- *
- * Revision 1.35  2001/07/24 23:40:15  craigs
- * Added ability to remove Q931 IE
- *
- * Revision 1.34  2001/06/14 06:25:16  robertj
- * Added further H.225 PDU build functions.
- * Moved some functionality from connection to PDU class.
- *
- * Revision 1.33  2001/05/30 04:38:40  robertj
- * Added BuildStatusEnquiry() Q.931 function, thanks Markus Storm
- *
- * Revision 1.32  2001/04/05 00:06:31  robertj
- * Fixed some more encoding/decoding problems with little used bits of
- *   the Q.931 protocol, thanks Hans Verbeek.
- *
- * Revision 1.31  2001/04/03 23:06:15  robertj
- * Fixed correct encoding and decoding of Q.850 cause field, thanks Hans Verbeek.
- *
- * Revision 1.30  2001/02/09 05:13:56  craigs
- * Added pragma implementation to (hopefully) reduce the executable image size
- * under Linux
- *
- * Revision 1.29  2001/01/19 06:57:26  robertj
- * Added Information message type.
- *
- * Revision 1.28  2000/10/13 02:16:04  robertj
- * Added support for Progress Indicator Q.931/H.225 message.
- *
- * Revision 1.27  2000/07/11 11:17:01  robertj
- * Improved trace log display of Q.931 PDU's (correct order and extra IE fields).
- *
- * Revision 1.26  2000/07/09 14:54:11  robertj
- * Added facility IE to facility message.
- * Changed reference to the word "field" to be more correct IE or "Information Element"
- *
- * Revision 1.25  2000/06/21 08:07:47  robertj
- * Added cause/reason to release complete PDU, where relevent.
- *
- * Revision 1.24  2000/05/09 12:19:31  robertj
- * Added ability to get and set "distinctive ring" Q.931 functionality.
- *
- * Revision 1.23  2000/05/08 14:07:35  robertj
- * Improved the provision and detection of calling and caller numbers, aliases and hostnames.
- *
- * Revision 1.22  2000/05/06 02:18:26  robertj
- * Changed the new CallingPartyNumber code so defaults for octet3a are application dependent.
- *
- * Revision 1.21  2000/05/05 00:44:05  robertj
- * Added presentation and screening fields to Calling Party Number field, thanks Dean Anderson.
- *
- * Revision 1.20  2000/05/02 04:32:27  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.19  2000/03/21 01:08:11  robertj
- * Fixed incorrect call reference code being used in originated call.
- *
- * Revision 1.18  2000/02/17 12:07:43  robertj
- * Used ne wPWLib random number generator after finding major problem in MSVC rand().
- *
- * Revision 1.17  1999/12/23 22:44:06  robertj
- * Added calling party number field.
- *
- * Revision 1.16  1999/09/22 04:18:29  robertj
- * Fixed missing "known" message types in debugging output.
- *
- * Revision 1.15  1999/09/10 03:36:48  robertj
- * Added simple Q.931 Status response to Q.931 Status Enquiry
- *
- * Revision 1.14  1999/08/31 13:54:35  robertj
- * Fixed problem with memory overrun building PDU's
- *
- * Revision 1.13  1999/08/31 12:34:19  robertj
- * Added gatekeeper support.
- *
- * Revision 1.12  1999/08/13 06:34:38  robertj
- * Fixed problem in CallPartyNumber Q.931 encoding.
- * Added field name display to Q.931 protocol.
- *
- * Revision 1.11  1999/08/10 13:14:15  robertj
- * Added Q.931 Called Number field if have "phone number" style destination addres.
- *
- * Revision 1.10  1999/07/16 02:15:30  robertj
- * Fixed more tunneling problems.
- *
- * Revision 1.9  1999/07/09 14:59:59  robertj
- * Fixed GNU C++ compatibility.
- *
- * Revision 1.8  1999/07/09 06:09:50  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.7  1999/06/14 15:19:48  robertj
- * GNU C compatibility
- *
- * Revision 1.6  1999/06/13 12:41:14  robertj
- * Implement logical channel transmitter.
- * Fixed H245 connect on receiving call.
- *
- * Revision 1.5  1999/06/09 05:26:20  robertj
- * Major restructuring of classes.
- *
- * Revision 1.4  1999/02/23 11:04:29  robertj
- * Added capability to make outgoing call.
- *
- * Revision 1.3  1999/01/16 01:31:38  robertj
- * Major implementation.
- *
- * Revision 1.2  1999/01/02 04:00:52  robertj
- * Added higher level protocol negotiations.
- *
- * Revision 1.1  1998/12/14 09:13:37  robertj
- * Initial revision
- *
+ * $Revision: 20836 $
+ * $Author: rjongbloed $
+ * $Date: 2008-09-03 05:24:34 +0000 (Wed, 03 Sep 2008) $
  */
 
 #include <ptlib.h>
 
+#include <opal/buildopts.h>
+#if OPAL_H323
+
 #ifdef __GNUC__
 #pragma implementation "q931.h"
 #endif
@@ -354,12 +134,13 @@ Q931::Q931()
 {
   protocolDiscriminator = 8;  // Q931 always has 00001000
   messageType = NationalEscapeMsg;
-  fromDestination = FALSE;
+  fromDestination = PFalse;
   callReference = 0;
 }
 
 
 Q931::Q931(const Q931 & other)
+  : PObject(other)
 {
   operator=(other);
 }
@@ -371,16 +152,14 @@ Q931 & Q931::operator=(const Q931 & other)
   fromDestination = other.fromDestination;
   protocolDiscriminator = other.protocolDiscriminator;
   messageType = other.messageType;
-
-  informationElements.RemoveAll();
-  for (PINDEX i = 0; i < other.informationElements.GetSize(); i++)
-    informationElements.SetAt(other.informationElements.GetKeyAt(i), new PBYTEArray(other.informationElements.GetDataAt(i)));
+  informationElements = other.informationElements;
+  informationElements.MakeUnique();
 
   return *this;
 }
 
 
-void Q931::BuildFacility(int callRef, BOOL fromDest)
+void Q931::BuildFacility(int callRef, PBoolean fromDest)
 {
   messageType = FacilityMsg;
   callReference = callRef;
@@ -391,7 +170,7 @@ void Q931::BuildFacility(int callRef, BOOL fromDest)
 }
 
 
-void Q931::BuildInformation(int callRef, BOOL fromDest)
+void Q931::BuildInformation(int callRef, PBoolean fromDest)
 {
   messageType = InformationMsg;
   callReference = callRef;
@@ -401,7 +180,7 @@ void Q931::BuildInformation(int callRef, BOOL fromDest)
 
 
 void Q931::BuildProgress(int callRef,
-                         BOOL fromDest,
+                         PBoolean fromDest,
                          unsigned description,
                          unsigned codingStandard,
                          unsigned location)
@@ -414,7 +193,7 @@ void Q931::BuildProgress(int callRef,
 }
 
 
-void Q931::BuildNotify(int callRef, BOOL fromDest)
+void Q931::BuildNotify(int callRef, PBoolean fromDest)
 {
   messageType = NotifyMsg;
   callReference = callRef;
@@ -427,7 +206,7 @@ void Q931::BuildSetupAcknowledge(int callRef)
 {
   messageType = SetupAckMsg;
   callReference = callRef;
-  fromDestination = TRUE;
+  fromDestination = PTrue;
   informationElements.RemoveAll();
 }
 
@@ -436,7 +215,7 @@ void Q931::BuildCallProceeding(int callRef)
 {
   messageType = CallProceedingMsg;
   callReference = callRef;
-  fromDestination = TRUE;
+  fromDestination = PTrue;
   informationElements.RemoveAll();
 }
 
@@ -445,7 +224,7 @@ void Q931::BuildAlerting(int callRef)
 {
   messageType = AlertingMsg;
   callReference = callRef;
-  fromDestination = TRUE;
+  fromDestination = PTrue;
   informationElements.RemoveAll();
 }
 
@@ -457,7 +236,7 @@ void Q931::BuildSetup(int callRef)
     callReference = GenerateCallReference();
   else
     callReference = callRef;
-  fromDestination = FALSE;
+  fromDestination = PFalse;
   informationElements.RemoveAll();
   SetBearerCapabilities(TransferSpeech, 1);
 }
@@ -467,13 +246,13 @@ void Q931::BuildConnect(int callRef)
 {
   messageType = ConnectMsg;
   callReference = callRef;
-  fromDestination = TRUE;
+  fromDestination = PTrue;
   informationElements.RemoveAll();
   SetBearerCapabilities(TransferSpeech, 1);
 }
 
 
-void Q931::BuildStatus(int callRef, BOOL fromDest)
+void Q931::BuildStatus(int callRef, PBoolean fromDest)
 {
   messageType = StatusMsg;
   callReference = callRef;
@@ -485,7 +264,7 @@ void Q931::BuildStatus(int callRef, BOOL fromDest)
 }
 
 
-void Q931::BuildStatusEnquiry(int callRef, BOOL fromDest)
+void Q931::BuildStatusEnquiry(int callRef, PBoolean fromDest)
 {
   messageType = StatusEnquiryMsg;
   callReference = callRef;
@@ -494,7 +273,7 @@ void Q931::BuildStatusEnquiry(int callRef, BOOL fromDest)
 }
 
 
-void Q931::BuildReleaseComplete(int callRef, BOOL fromDest)
+void Q931::BuildReleaseComplete(int callRef, PBoolean fromDest)
 {
   messageType = ReleaseCompleteMsg;
   callReference = callRef;
@@ -503,18 +282,18 @@ void Q931::BuildReleaseComplete(int callRef, BOOL fromDest)
 }
 
 
-BOOL Q931::Decode(const PBYTEArray & data)
+PBoolean Q931::Decode(const PBYTEArray & data)
 {
   // Clear all existing data before reading new
   informationElements.RemoveAll();
 
   if (data.GetSize() < 5) // Packet too short
-    return FALSE;
+    return PFalse;
 
   protocolDiscriminator = data[0];
 
   if (data[1] != 2) // Call reference must be 2 bytes long
-    return FALSE;
+    return PFalse;
 
   callReference = ((data[2]&0x7f) << 8) | data[3];
   fromDestination = (data[2]&0x80) != 0;
@@ -525,12 +304,12 @@ BOOL Q931::Decode(const PBYTEArray & data)
   PINDEX offset = 5;
   while (offset < data.GetSize()) {
     // Get field discriminator
-    int discriminator = data[offset++];
-
-    PBYTEArray * item = new PBYTEArray;
+    InformationElementCodes discriminator = (InformationElementCodes)data[offset++];
 
     // For discriminator with high bit set there is no data
-    if ((discriminator&0x80) == 0) {
+    if ((discriminator&0x80) != 0)
+      SetIE(discriminator, PBYTEArray(), TRUE);
+    else {
       int len = data[offset++];
 
       if (discriminator == UserUserIE) {
@@ -543,42 +322,43 @@ BOOL Q931::Decode(const PBYTEArray & data)
 
 	// before decrementing the length, make sure it is not zero
 	if (len == 0)
-          return FALSE;
+          return PFalse;
 
         // adjust for protocol discriminator
         len--;
       }
 
       if (offset + len > data.GetSize())
-        return FALSE;
+        return PFalse;
+
+      SetIE(discriminator, PBYTEArray((const BYTE *)data+offset, len), TRUE);
 
-      memcpy(item->GetPointer(len), (const BYTE *)data+offset, len);
       offset += len;
     }
-
-    informationElements.SetAt(discriminator, item);
   }
 
-  return TRUE;
+  return PTrue;
 }
 
 
-BOOL Q931::Encode(PBYTEArray & data) const
+PBoolean Q931::Encode(PBYTEArray & data) const
 {
   PINDEX totalBytes = 5;
   unsigned discriminator;
   for (discriminator = 0; discriminator < 256; discriminator++) {
     if (informationElements.Contains(discriminator)) {
-      if (discriminator < 128)
-        totalBytes += informationElements[discriminator].GetSize() +
-                            (discriminator != UserUserIE ? 2 : 4);
-      else
-        totalBytes++;
+      const InternalInformationElement & element = informationElements[discriminator];
+      for (PINDEX idx = 0; idx < element.GetSize(); idx++) {
+        if (discriminator < 128)
+          totalBytes += element[idx].GetSize() + (discriminator != UserUserIE ? 2 : 4);
+        else
+          totalBytes++;
+      }
     }
   }
 
   if (!data.SetMinSize(totalBytes))
-    return FALSE;
+    return PFalse;
 
   // Put in Q931 header
   PAssert(protocolDiscriminator < 256, PInvalidParameter);
@@ -596,28 +376,31 @@ BOOL Q931::Encode(PBYTEArray & data) const
   PINDEX offset = 5;
   for (discriminator = 0; discriminator < 256; discriminator++) {
     if (informationElements.Contains(discriminator)) {
-      if (discriminator < 128) {
-        int len = informationElements[discriminator].GetSize();
+      const InternalInformationElement & element = informationElements[discriminator];
+      for (PINDEX i = 0; i < element.GetSize(); i++) {
+        if (discriminator < 128) {
+          int len = element[i].GetSize();
 
-        if (discriminator != UserUserIE) {
-          data[offset++] = (BYTE)discriminator;
-          data[offset++] = (BYTE)len;
-        }
-        else {
-          len++; // Allow for protocol discriminator
-          data[offset++] = (BYTE)discriminator;
-          data[offset++] = (BYTE)(len >> 8);
-          data[offset++] = (BYTE)len;
-          len--; // Then put the length back again
-          // We shall assume that the user-user field is an ITU protocol block (5)
-          data[offset++] = 5;
-        }
+          if (discriminator != UserUserIE) {
+            data[offset++] = (BYTE)discriminator;
+            data[offset++] = (BYTE)len;
+          }
+          else {
+            len++; // Allow for protocol discriminator
+            data[offset++] = (BYTE)discriminator;
+            data[offset++] = (BYTE)(len >> 8);
+            data[offset++] = (BYTE)len;
+            len--; // Then put the length back again
+            // We shall assume that the user-user field is an ITU protocol block (5)
+            data[offset++] = 5;
+          }
 
-        memcpy(&data[offset], (const BYTE *)informationElements[discriminator], len);
-        offset += len;
+          memcpy(&data[offset], (const BYTE *)element[i], len);
+          offset += len;
+        }
+        else
+          data[offset++] = (BYTE)discriminator;
       }
-      else
-        data[offset++] = (BYTE)discriminator;
     }
   }
 
@@ -628,7 +411,7 @@ BOOL Q931::Encode(PBYTEArray & data) const
 void Q931::PrintOn(ostream & strm) const
 {
   int indent = strm.precision() + 2;
-  _Ios_Fmtflags flags = strm.flags();
+  ios::fmtflags flags = strm.flags();
 
   strm << "{\n"
        << setw(indent+24) << "protocolDiscriminator = " << protocolDiscriminator << '\n'
@@ -638,28 +421,31 @@ void Q931::PrintOn(ostream & strm) const
 
   for (unsigned discriminator = 0; discriminator < 256; discriminator++) {
     if (informationElements.Contains(discriminator)) {
-      strm << setw(indent+4) << "IE: " << (InformationElementCodes)discriminator;
-      if (discriminator == CauseIE) {
-        if (informationElements[discriminator].GetSize() > 1)
-          strm << " - " << (CauseValues)(informationElements[discriminator][1]&0x7f);
-      }
-      strm << " = {\n"
-           << hex << setfill('0') << resetiosflags(ios::floatfield)
-           << setprecision(indent+2) << setw(16);
+      const InternalInformationElement & element = informationElements[discriminator];
+      for (PINDEX idx = 0; idx < element.GetSize(); idx++) {
+        strm << setw(indent+4) << "IE: " << (InformationElementCodes)discriminator;
+        if (discriminator == CauseIE) {
+          if (element[idx].GetSize() > 1)
+            strm << " - " << (CauseValues)(element[idx][1]&0x7f);
+        }
+        strm << " = {\n"
+             << hex << setfill('0') << resetiosflags(ios::floatfield)
+             << setprecision(indent+2) << setw(16);
 
-      PBYTEArray value = informationElements[discriminator];
-      if (value.GetSize() <= 32 || (flags&ios::floatfield) != ios::fixed)
-        strm << value;
-      else {
-        PBYTEArray truncatedArray(value, 32);
-        strm << truncatedArray << '\n'
-             << setfill(' ')
-             << setw(indent+5) << "...";
-      }
+        PBYTEArray value = element[idx];
+        if (value.GetSize() <= 32 || (flags&ios::floatfield) != ios::fixed)
+          strm << value;
+        else {
+          PBYTEArray truncatedArray(value, 32);
+          strm << truncatedArray << '\n'
+               << setfill(' ')
+               << setw(indent+5) << "...";
+        }
 
-      strm << dec << setfill(' ')
-           << '\n'
-           << setw(indent+2) << "}\n";
+        strm << dec << setfill(' ')
+             << '\n'
+             << setw(indent+2) << "}\n";
+      }
     }
   }
 
@@ -698,6 +484,34 @@ PString Q931::GetMessageTypeName() const
       return "Information";
     case NationalEscapeMsg :
       return "Escape";
+    case NotifyMsg :
+      return "NotifyMsg";
+    case ResumeMsg :
+      return "ResumeMsg";
+    case ResumeAckMsg :
+      return "ResumeAckMsg";
+    case ResumeRejectMsg :
+      return "ResumeRejectMsg";
+    case SuspendMsg :
+      return "SuspendMsg";
+    case SuspendAckMsg :
+      return "SuspendAckMsg";
+    case SuspendRejectMsg :
+      return "SuspendRejectMsg";
+    case UserInformationMsg :
+      return "UserInformationMsg";
+    case DisconnectMsg :
+      return "DisconnectMsg";
+    case ReleaseMsg :
+      return "ReleaseMsg";
+    case RestartMsg :
+      return "RestartMsg";
+    case RestartAckMsg :
+      return "RestartAckMsg";
+    case SegmentMsg :
+      return "SegmentMsg";
+    case CongestionCtrlMsg :
+      return "CongestionCtrlMsg";
     default :
       break;
   }
@@ -726,24 +540,30 @@ unsigned Q931::GenerateCallReference()
 }
 
 
-BOOL Q931::HasIE(InformationElementCodes ie) const
+PBoolean Q931::HasIE(InformationElementCodes ie) const
 {
   return informationElements.Contains(POrdinalKey(ie));
 }
 
 
-PBYTEArray Q931::GetIE(InformationElementCodes ie) const
+PBYTEArray Q931::GetIE(InformationElementCodes ie, PINDEX idx) const
 {
   if (informationElements.Contains(POrdinalKey(ie)))
-    return informationElements[ie];
+    return informationElements[ie][idx];
 
   return PBYTEArray();
 }
 
 
-void Q931::SetIE(InformationElementCodes ie, const PBYTEArray & userData)
+void Q931::SetIE(InformationElementCodes ie, const PBYTEArray & userData, bool append)
 {
-  informationElements.SetAt(ie, new PBYTEArray(userData));
+  if (append && informationElements.Contains(ie))
+    informationElements[ie].Append(new PBYTEArray(userData));
+  else {
+    InternalInformationElement * element = new InternalInformationElement;
+    element->Append(new PBYTEArray(userData));
+    informationElements.SetAt(ie, element);
+  }
 }
 
 void Q931::RemoveIE(InformationElementCodes ie)
@@ -806,17 +626,17 @@ void Q931::SetBearerCapabilities(InformationTransferCapability capability,
 }
 
 
-BOOL Q931::GetBearerCapabilities(InformationTransferCapability & capability,
+PBoolean Q931::GetBearerCapabilities(InformationTransferCapability & capability,
                                  unsigned & transferRate,
                                  unsigned * codingStandard,
                                  unsigned * userInfoLayer1)
 {
   if (!HasIE(BearerCapabilityIE))
-    return FALSE;
+    return PFalse;
 
   PBYTEArray data = GetIE(BearerCapabilityIE);
   if (data.GetSize() < 2)
-    return FALSE;
+    return PFalse;
 
   capability = (InformationTransferCapability)data[0];
   if (codingStandard != NULL)
@@ -841,18 +661,18 @@ BOOL Q931::GetBearerCapabilities(InformationTransferCapability & capability,
       break;
     case 0x18 :
       if (data.GetSize() < 3)
-        return FALSE;
+        return PFalse;
       transferRate = data[2]&0x7f;
       nextByte = 3;
       break;
     default :
-      return FALSE;
+      return PFalse;
   }
 
   if (userInfoLayer1 != NULL)
     *userInfoLayer1 = data.GetSize() >= nextByte && ((data[nextByte]>>5)&3) == 1 ? (data[nextByte]&0x1f) : 0;
 
-  return TRUE;
+  return PTrue;
 }
 
 
@@ -970,16 +790,16 @@ void Q931::SetProgressIndicator(unsigned description,
 }
 
 
-BOOL Q931::GetProgressIndicator(unsigned & description,
+PBoolean Q931::GetProgressIndicator(unsigned & description,
                                 unsigned * codingStandard,
                                 unsigned * location) const
 {
   if (!HasIE(ProgressIndicatorIE))
-    return FALSE;
+    return PFalse;
 
   PBYTEArray data = GetIE(ProgressIndicatorIE);
   if (data.GetSize() < 2)
-    return FALSE;
+    return PFalse;
 
   if (codingStandard != NULL)
     *codingStandard = (data[0]>>5)&0x03;
@@ -987,7 +807,7 @@ BOOL Q931::GetProgressIndicator(unsigned & description,
     *location = data[0]&0x0f;
   description = data[1]&0x7f;
 
-  return TRUE;
+  return PTrue;
 }
 
 
@@ -1056,7 +876,7 @@ static PBYTEArray SetNumberIE(const PString & number,
 }
 
 
-static BOOL GetNumberIE(const PBYTEArray & bytes,
+static PBoolean GetNumberIE(const PBYTEArray & bytes,
                         PString  & number,
                         unsigned * plan,
                         unsigned * type,
@@ -1070,7 +890,7 @@ static BOOL GetNumberIE(const PBYTEArray & bytes,
   number = PString();
 
   if (bytes.IsEmpty())
-    return FALSE;
+    return PFalse;
 
   if (plan != NULL)
     *plan = bytes[0]&15;
@@ -1090,7 +910,7 @@ static BOOL GetNumberIE(const PBYTEArray & bytes,
   }
   else {
     if (bytes.GetSize() < 2)
-      return FALSE;
+      return PFalse;
 
     if (presentation != NULL)
       *presentation = (bytes[1]>>5)&3;
@@ -1106,7 +926,7 @@ static BOOL GetNumberIE(const PBYTEArray & bytes,
     }
     else {
       if (bytes.GetSize() < 3)
-        return FALSE;
+        return PFalse;
 
       if (reason != NULL)
         *reason = bytes[2]&15;
@@ -1116,7 +936,7 @@ static BOOL GetNumberIE(const PBYTEArray & bytes,
   }
 
   if (bytes.GetSize() < offset)
-    return FALSE;
+    return PFalse;
 
   PINDEX len = bytes.GetSize()-offset;
 
@@ -1138,7 +958,7 @@ void Q931::SetCallingPartyNumber(const PString & number,
 }
 
 
-BOOL Q931::GetCallingPartyNumber(PString  & number,
+PBoolean Q931::GetCallingPartyNumber(PString  & number,
                                  unsigned * plan,
                                  unsigned * type,
                                  unsigned * presentation,
@@ -1159,7 +979,7 @@ void Q931::SetCalledPartyNumber(const PString & number, unsigned plan, unsigned
 }
 
 
-BOOL Q931::GetCalledPartyNumber(PString & number, unsigned * plan, unsigned * type) const
+PBoolean Q931::GetCalledPartyNumber(PString & number, unsigned * plan, unsigned * type) const
 {
   return GetNumberIE(GetIE(CalledPartyNumberIE),
                      number, plan, type, NULL, NULL, NULL, 0, 0, 0);
@@ -1178,7 +998,7 @@ void Q931::SetRedirectingNumber(const PString & number,
 }
 
 
-BOOL Q931::GetRedirectingNumber(PString  & number,
+PBoolean Q931::GetRedirectingNumber(PString  & number,
                                 unsigned * plan,
                                 unsigned * type,
                                 unsigned * presentation,
@@ -1194,7 +1014,7 @@ BOOL Q931::GetRedirectingNumber(PString  & number,
 }
 
 
-BOOL Q931::GetConnectedNumber(PString  & number,
+PBoolean Q931::GetConnectedNumber(PString  & number,
                               unsigned * plan,
                               unsigned * type,
                               unsigned * presentation,
@@ -1242,7 +1062,7 @@ void Q931::SetChannelIdentification(unsigned interfaceType,
       bytes[0] = 0x80 | 0x04 | 0x03;
     }
     if (channelNumber == 0) { // D channel
-      bytes[0] = 0x80;
+      bytes[0] = 0x80 | 0x08 | 0x04;
     }    
     if (channelNumber > 0) { // B channel
       bytes[0] = (BYTE)(0x80 | 0x04 | ((preferredOrExclusive & 0x01) << 3) | (channelNumber & 0x03));
@@ -1254,7 +1074,7 @@ void Q931::SetChannelIdentification(unsigned interfaceType,
       bytes[0] = 0x80 | 0x20 | 0x04 | 0x03;
     }
     if (channelNumber == 0) { // D channel
-      bytes[0] = 0x80 | 0x20;
+      bytes[0] = 0x80 | 0x20 | 0x08 | 0x04;
     }    
     if (channelNumber > 0) { // B channel
       bytes.SetSize(3);
@@ -1269,16 +1089,16 @@ void Q931::SetChannelIdentification(unsigned interfaceType,
 }
 
 
-BOOL Q931::GetChannelIdentification(unsigned * interfaceType,
+PBoolean Q931::GetChannelIdentification(unsigned * interfaceType,
                                     unsigned * preferredOrExclusive,
                                     int      * channelNumber) const
 {
   if (!HasIE(ChannelIdentificationIE))
-    return FALSE;
+    return PFalse;
 
   PBYTEArray bytes = GetIE(ChannelIdentificationIE);
   if (bytes.GetSize() < 1)
-    return FALSE;
+    return PFalse;
 
   *interfaceType        = (bytes[0]>>5) & 0x01;
   *preferredOrExclusive = (bytes[0]>>3) & 0x01;
@@ -1307,18 +1127,20 @@ BOOL Q931::GetChannelIdentification(unsigned * interfaceType,
       }
       else { // B Channel
         if (bytes.GetSize() < 3)
-          return FALSE;
+          return PFalse;
 
         if (bytes[1] != 0x83)
-          return FALSE;
+          return PFalse;
 
         *channelNumber = bytes[2] & 0x7f;
       }
     }
   }
 
-  return TRUE;
+  return PTrue;
 }
 
 
+#endif // OPAL_H323
+
 /////////////////////////////////////////////////////////////////////////////
