@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21851 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 01:35:12 +0000 (Mon, 22 Dec 2008) $
+ * $Revision: 22097 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-21 04:55:10 +0000 (Sat, 21 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -66,6 +66,7 @@ ostream & operator<<(ostream & out, OpalConnection::Phases phase)
   static const char * const names[OpalConnection::NumPhases] = {
     "UninitialisedPhase",
     "SetUpPhase",
+    "ProceedingPhase",
     "AlertingPhase",
     "ConnectedPhase",
     "EstablishedPhase",
@@ -407,7 +408,7 @@ PBoolean OpalConnection::OnIncomingConnection(unsigned options, OpalConnection::
 
 PString OpalConnection::GetDestinationAddress()
 {
-  return "*";
+  return PString::Empty();
 }
 
 
@@ -423,6 +424,11 @@ PSafePtr<OpalConnection> OpalConnection::GetOtherPartyConnection() const
 }
 
 
+void OpalConnection::OnProceeding()
+{
+  endpoint.OnProceeding(*this);
+}
+
 void OpalConnection::OnAlerting()
 {
   endpoint.OnAlerting(*this);
@@ -612,8 +618,6 @@ void OpalConnection::StartMediaStreams()
 
 void OpalConnection::CloseMediaStreams()
 {
-  GetCall().OnStopRecordAudio(callIdentifier.AsString());
-
   // Do this double loop as while closing streams, the instance may disappear from the
   // mediaStreams list, prematurely stopping the for loop.
   bool someOpen = true;
@@ -694,21 +698,35 @@ PBoolean OpalConnection::OnOpenMediaStream(OpalMediaStream & stream)
 }
 
 
+static PString MakeRecordingKey(const OpalMediaPatch & patch)
+{
+  return psprintf("%08x", &patch);
+}
+
+
 void OpalConnection::OnClosedMediaStream(const OpalMediaStream & stream)
 {
+  OpalMediaPatch * patch = stream.GetPatch();
+  if (patch != NULL)
+    GetCall().OnStopRecordAudio(MakeRecordingKey(*patch));
+
   endpoint.OnClosedMediaStream(stream);
 }
 
 
-void OpalConnection::OnPatchMediaStream(PBoolean PTRACE_PARAM(isSource), OpalMediaPatch & PTRACE_PARAM(patch))
+void OpalConnection::OnPatchMediaStream(PBoolean PTRACE_PARAM(isSource), OpalMediaPatch & patch)
 {
   if (!recordAudioFilename.IsEmpty())
     GetCall().StartRecording(recordAudioFilename);
+  else if (GetCall().IsRecording()) {
+    patch.AddFilter(recordNotifier, OPAL_PCM16);
+    PTRACE(4, "OpalCon\tAdded record filter on connection " << *this << ", patch " << patch);
+  }
 
-  // TODO - add autorecord functions here
   PTRACE(3, "OpalCon\t" << (isSource ? "Source" : "Sink") << " stream of connection " << *this << " uses patch " << patch);
 }
 
+
 void OpalConnection::EnableRecording()
 {
   if (!LockReadWrite())
@@ -717,13 +735,16 @@ void OpalConnection::EnableRecording()
   OpalMediaStreamPtr stream = GetMediaStream(OpalMediaType::Audio(), true);
   if (stream != NULL) {
     OpalMediaPatch * patch = stream->GetPatch();
-    if (patch != NULL)
+    if (patch != NULL) {
       patch->AddFilter(recordNotifier, OPAL_PCM16);
+      PTRACE(4, "OpalCon\tAdded record filter on " << MakeRecordingKey(*patch));
+    }
   }
   
   UnlockReadWrite();
 }
 
+
 void OpalConnection::DisableRecording()
 {
   if (!LockReadWrite())
@@ -732,18 +753,23 @@ void OpalConnection::DisableRecording()
   OpalMediaStreamPtr stream = GetMediaStream(OpalMediaType::Audio(), true);
   if (stream != NULL) {
     OpalMediaPatch * patch = stream->GetPatch();
-    if (patch != NULL)
+    if (patch != NULL) {
       patch->RemoveFilter(recordNotifier, OPAL_PCM16);
+      PTRACE(4, "OpalCon\tRemoved record filter on " << MakeRecordingKey(*patch));
+    }
   }
   
   UnlockReadWrite();
 }
 
-void OpalConnection::OnRecordAudio(RTP_DataFrame & frame, INT)
+
+void OpalConnection::OnRecordAudio(RTP_DataFrame & frame, INT param)
 {
-  GetCall().GetManager().GetRecordManager().WriteAudio(GetCall().GetToken(), callIdentifier.AsString(), frame);
+  const OpalMediaPatch * patch = (const OpalMediaPatch *)param;
+  GetCall().OnRecordAudio(MakeRecordingKey(*patch), frame);
 }
 
+
 void OpalConnection::AttachRFC2833HandlerToPatch(PBoolean /*isSource*/, OpalMediaPatch & /*patch*/)
 {
 }
@@ -1045,6 +1071,18 @@ void OpalConnection::CopyPartyNames(const OpalConnection & other)
 }
 
 
+PString OpalConnection::GetAlertingType() const
+{
+  return PString::Empty();
+}
+
+
+bool OpalConnection::SetAlertingType(const PString & /*info*/)
+{
+  return false;
+}
+
+
 void OpalConnection::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
 {
   maxDelay = PMAX(10, PMIN(maxDelay, 999));
@@ -1057,6 +1095,13 @@ void OpalConnection::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
   maxAudioJitterDelay = maxDelay;
 }
 
+
+PString OpalConnection::GetIdentifier() const
+{
+  return GetToken();
+}
+
+
 PINDEX OpalConnection::GetMaxRtpPayloadSize() const
 { 
   return endpoint.GetManager().GetMaxRtpPayloadSize(); 
@@ -1094,15 +1139,13 @@ void OpalConnection::OnApplyStringOptions()
 
 void OpalConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
 {
+  PTRACE(4, "OpalCon\tApplying string options:\n" << stringOptions);
+
   if (LockReadWrite()) {
 
     m_connStringOptions = stringOptions;
   
-    PString str(stringOptions("Call-Identifier"));
-    if (!str.IsEmpty())
-      callIdentifier = PGloballyUniqueID(str);
-
-    str = stringOptions("enableinbanddtmf");
+    PCaselessString str = stringOptions("enableinbanddtmf");
     if (!str.IsEmpty())
       detectInBandDTMF = str *= "true";
     str = stringOptions("dtmfmult");
@@ -1127,6 +1170,10 @@ void OpalConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOp
     if (stringOptions.Contains("Record-Audio"))
       recordAudioFilename = m_connStringOptions("Record-Audio");
 
+    str = stringOptions("Alerting-Type");
+    if (!str.IsEmpty())
+      SetAlertingType(str);
+
     UnlockReadWrite();
   }
 }
