@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22096 $
+ * $Revision: 22166 $
  * $Author: rjongbloed $
- * $Date: 2009-02-21 03:25:23 +0000 (Sat, 21 Feb 2009) $
+ * $Date: 2009-03-06 00:29:00 +0000 (Fri, 06 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -600,6 +600,10 @@ bool OpalManager::OnRouteConnection(PStringSet & routesTried,
   for (;;) {
     PString route = ApplyRouteTable(a_party, b_party, tableEntry);
     if (route.IsEmpty()) {
+      // Check for if B-Party is an explicit address
+      if (FindEndPoint(b_party.Left(b_party.Find(':'))) != NULL)
+        return MakeConnection(call, b_party, NULL, options, stringOptions);
+
       PTRACE(3, "OpalMan\tCould not route a=\"" << a_party << "\", b=\"" << b_party << ", call=" << call);
       return false;
     }
