@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 22122 $
+ * $Author: csoutheren $
+ * $Date: 2009-02-26 11:44:57 +0000 (Thu, 26 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -320,8 +320,10 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
     unsigned oldTimestamp = timestamp;
 
     PINDEX written;
-    if (!WriteData(ptr, size, written) || (written == 0))
+    if (!WriteData(ptr, size, written) || (written == 0)) {
+      PTRACE(2, "Media\tWritePacket failed with written " << written);
       return false;
+    }
 
     // If the Write() function did not change the timestamp then use the default
     // method of fixed frame times and sizes.
@@ -342,12 +344,16 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
 
 PBoolean OpalMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & length)
 {
-  if (!isOpen)
+  if (!isOpen) {
+    length = 0;
     return false;
+  }
 
   RTP_DataFrame packet(size);
-  if (!ReadPacket(packet))
+  if (!ReadPacket(packet)) {
+    length = 0;
     return false;
+  }
 
   length = packet.GetPayloadSize();
   if (length > size)
@@ -361,8 +367,10 @@ PBoolean OpalMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & length)
 
 PBoolean OpalMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX & written)
 {
-  if (!isOpen)
+  if (!isOpen) {
+    written = 0;
     return false;
+  }
 
   written = length;
   RTP_DataFrame packet(length);
@@ -389,6 +397,7 @@ PBoolean OpalMediaStream::SetDataSize(PINDEX dataSize)
   if (dataSize <= 0)
     return false;
 
+  PTRACE_IF(4, defaultDataSize != dataSize, "Media\tSet data size from " << defaultDataSize << " to " << dataSize);
   defaultDataSize = dataSize;
   return true;
 }
@@ -695,16 +704,17 @@ void OpalRTPMediaStream::GetStatistics(OpalMediaStatistics & statistics) const
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalRawMediaStream::OpalRawMediaStream(OpalConnection & conn,
-                                     const OpalMediaFormat & mediaFormat,
+                                       const OpalMediaFormat & mediaFormat,
                                        unsigned sessionID,
                                        PBoolean isSource,
-                                       PChannel * chan, PBoolean autoDel)
+                                       PChannel * chan, PBoolean autoDelete)
   : OpalMediaStream(conn, mediaFormat, sessionID, isSource)
+  , m_channel(chan)
+  , m_autoDelete(autoDelete)
+  , m_silence(160) // At least 10ms
+  , m_averageSignalSum(0)
+  , m_averageSignalSamples(0)
 {
-  channel = chan;
-  autoDelete = autoDel;
-  averageSignalSum = 0;
-  averageSignalSamples = 0;
 }
 
 
@@ -712,9 +722,9 @@ OpalRawMediaStream::~OpalRawMediaStream()
 {
   Close();
 
-  if (autoDelete)
-    delete channel;
-  channel = NULL;
+  if (m_autoDelete)
+    delete m_channel;
+  m_channel = NULL;
 }
 
 
@@ -730,22 +740,29 @@ PBoolean OpalRawMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & lengt
     return false;
   }
 
-  if (!IsOpen() || channel == NULL)
+  PWaitAndSignal mutex(m_channelMutex);
+
+  if (!IsOpen() || m_channel == NULL)
     return false;
 
-  if (!channel->Read(buffer, size))
+  if (!m_channel->Read(buffer, size))
     return false;
 
-  length = channel->GetLastReadCount();
-  CollectAverage(buffer, length);
+  length = m_channel->GetLastReadCount();
+
+  if (buffer != NULL)
+    CollectAverage(buffer, length);
+
   return true;
 }
 
 
 PBoolean OpalRawMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX & written)
 {
-  if (!isOpen)
+  if (!isOpen) {
+    PTRACE(1, "Media\tTried to write to closed media stream");
     return false;
+  }
 
   written = 0;
 
@@ -754,23 +771,61 @@ PBoolean OpalRawMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDE
     return false;
   }
   
-  if (!IsOpen() || channel == NULL)
+  PWaitAndSignal mutex(m_channelMutex);
+
+  if (!IsOpen() || m_channel == NULL) {
+    PTRACE(1, "Media\tTried to write to media stream with no channel");
+    return false;
+  }
+
+  /* We make the assumption that the remote is sending the same sized packets
+     all the time and does not suddenly switch from 30ms to 20ms, even though
+     this is technically legal. We have never yet seen it, and even if it did
+     it doesn't hurt the silence insertion algorithm very much.
+
+     So, silence buffer is set to be the largest chunk of audio the remote has
+     ever sent to us. Then when they stop sending, (we get length==0) we just
+     keep outputting that number of bytes to the raw channel until the remote
+     starts up again.
+     */
+
+  if (buffer != NULL && length != 0)
+    m_silence.SetMinSize(length);
+  else {
+    length = m_silence.GetSize();
+    buffer = m_silence;
+  }
+
+  if (!m_channel->Write(buffer, length))
     return false;
 
-  if (buffer != NULL && length != 0) {
-    if (!channel->Write(buffer, length))
-      return false;
-    written = channel->GetLastWriteCount();
-    CollectAverage(buffer, written);
-  }
-  else {
-    PBYTEArray silence(defaultDataSize);
-    if (!channel->Write(silence, defaultDataSize))
-      return false;
-    written = channel->GetLastWriteCount();
-    CollectAverage(silence, written);
+  written = m_channel->GetLastWriteCount();
+  CollectAverage(buffer, written);
+  return true;
+}
+
+
+bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
+{
+  if (chan == NULL || !chan->IsOpen()) {
+    if (autoDelete)
+      delete chan;
+    return false;
   }
 
+  m_channelMutex.Wait();
+
+  PChannel * channelToDelete = m_autoDelete ? m_channel : NULL;
+  m_channel = chan;
+  m_autoDelete = autoDelete;
+
+  SetDataSize(GetDataSize());
+
+  m_channelMutex.Signal();
+
+  delete channelToDelete; // Outside mutex
+
+  PTRACE(4, "Media\tSet raw media channel to \"" << m_channel->GetName() << '"');
   return true;
 }
 
@@ -780,8 +835,8 @@ PBoolean OpalRawMediaStream::Close()
   if (!OpalMediaStream::Close())
     return false;
 
-  if (channel != NULL)
-    channel->Close();
+  if (m_channel != NULL)
+    m_channel->Close();
 
   return true;
 }
@@ -789,27 +844,27 @@ PBoolean OpalRawMediaStream::Close()
 
 unsigned OpalRawMediaStream::GetAverageSignalLevel()
 {
-  PWaitAndSignal m(averagingMutex);
+  PWaitAndSignal mutex(m_averagingMutex);
 
-  if (averageSignalSamples == 0)
+  if (m_averageSignalSamples == 0)
     return UINT_MAX;
 
-  unsigned average = (unsigned)(averageSignalSum/averageSignalSamples);
-  averageSignalSum = average;
-  averageSignalSamples = 1;
+  unsigned average = (unsigned)(m_averageSignalSum/m_averageSignalSamples);
+  m_averageSignalSum = average;
+  m_averageSignalSamples = 1;
   return average;
 }
 
 
 void OpalRawMediaStream::CollectAverage(const BYTE * buffer, PINDEX size)
 {
-  PWaitAndSignal m(averagingMutex);
+  PWaitAndSignal mutex(m_averagingMutex);
 
   size = size/2;
-  averageSignalSamples += size;
+  m_averageSignalSamples += size;
   const short * pcm = (const short *)buffer;
   while (size-- > 0) {
-    averageSignalSum += PABS(*pcm);
+    m_averageSignalSum += PABS(*pcm);
     pcm++;
   }
 }
@@ -911,7 +966,7 @@ PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize)
   /* For efficiency reasons we will not accept a packet size that is too small.
      We move it up to the next even multiple, which has a danger of the remote not
      sending an even number of our multiplier. */
-  const unsigned MinTime = 20;
+  const unsigned MinTime = 10;
   PINDEX minSize = mediaFormat.GetClockRate()/(1000/MinTime)*sizeof(short);
   if (dataSize < minSize) {
     PTRACE(1, "Media\tClamping audio stream data size from " << dataSize << " to minimum " << minSize);
@@ -921,7 +976,7 @@ PBoolean OpalAudioMediaStream::SetDataSize(PINDEX dataSize)
   PTRACE(3, "Media\tAudio " << (IsSource() ? "source" : "sink") << " data size set to "
          << dataSize << " bytes and " << soundChannelBuffers << " buffers.");
   return OpalMediaStream::SetDataSize(dataSize) &&
-         ((PSoundChannel *)channel)->SetBuffers(dataSize, soundChannelBuffers);
+         ((PSoundChannel *)m_channel)->SetBuffers(dataSize, soundChannelBuffers);
 }
 
 
@@ -1073,7 +1128,7 @@ PBoolean OpalVideoMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
   marker = true;
   length = bytesReturned + sizeof(PluginCodec_Video_FrameHeader);
 
-  if ((flags & PluginCodec_CoderForceIFrame) != 0) {
+  if ((flags & PluginCodec_ReturnCoderRequestIFrame) != 0) {
     ExecuteCommand(OpalVideoUpdatePicture());
   }
 
