@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 21115 $
+ * $Revision: 22197 $
  * $Author: rjongbloed $
- * $Date: 2008-09-22 03:00:58 +0000 (Mon, 22 Sep 2008) $
+ * $Date: 2009-03-12 01:11:14 +0000 (Thu, 12 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -79,7 +79,7 @@ OpalTransportAddress::OpalTransportAddress()
 OpalTransportAddress::OpalTransportAddress(const char * cstr,
                                            WORD port,
                                            const char * proto)
-  : PString(cstr)
+  : PCaselessString(cstr)
 {
   SetInternalTransport(port, proto);
 }
@@ -88,14 +88,14 @@ OpalTransportAddress::OpalTransportAddress(const char * cstr,
 OpalTransportAddress::OpalTransportAddress(const PString & str,
                                            WORD port,
                                            const char * proto)
-  : PString(str)
+  : PCaselessString(str)
 {
   SetInternalTransport(port, proto);
 }
 
 
 OpalTransportAddress::OpalTransportAddress(const PIPSocket::Address & addr, WORD port, const char * proto)
-  : PString(addr.AsString())
+  : PCaselessString(addr.IsAny() ? PString('*') : addr.AsString(true))
 {
   SetInternalTransport(port, proto);
 }
@@ -108,9 +108,9 @@ PString OpalTransportAddress::GetHostName() const
 
   return transport->GetHostName(*this);
 }
-  
 
-PBoolean OpalTransportAddress::IsEquivalent(const OpalTransportAddress & address) const
+
+PBoolean OpalTransportAddress::IsEquivalent(const OpalTransportAddress & address, bool wildcard) const
 {
   if (*this == address)
     return PTrue;
@@ -122,8 +122,8 @@ PBoolean OpalTransportAddress::IsEquivalent(const OpalTransportAddress & address
   WORD port1 = 65535, port2 = 65535;
   return GetIpAndPort(ip1, port1) &&
          address.GetIpAndPort(ip2, port2) &&
-         (ip1.IsAny() || ip2.IsAny() || (ip1 *= ip2)) &&
-         (port1 == 65535 || port2 == 65535 || port1 == port2);
+         ((ip1 *= ip2) || (wildcard && (ip1.IsAny() || ip2.IsAny()))) &&
+         (port1 == port2 || (wildcard && (port1 == 65535 || port2 == 65535)));
 }
 
 
@@ -132,8 +132,8 @@ PBoolean OpalTransportAddress::IsCompatible(const OpalTransportAddress & address
   if (IsEmpty() || address.IsEmpty())
     return PTrue;
 
-  PCaselessString myPrefix = Left(Find('$'));
-  PCaselessString theirPrefix = address.Left(address.Find('$'));
+  PCaselessString myPrefix = GetProto();
+  PCaselessString theirPrefix = address.GetProto();
   return myPrefix == theirPrefix ||
         (myPrefix    == IpPrefix && (theirPrefix == TcpPrefix || theirPrefix == UdpPrefix 
 #if OPAL_PTLIB_SSL
@@ -167,6 +167,21 @@ PBoolean OpalTransportAddress::GetIpAndPort(PIPSocket::Address & ip, WORD & port
 }
 
 
+PBoolean OpalTransportAddress::GetIpAndPort(PIPSocketAddressAndPort & ipPort) const
+{
+  if (transport == NULL)
+    return PFalse;
+
+  PIPSocket::Address ip;
+  WORD port = 0;
+  if (!transport->GetIpAndPort(*this, ip, port))
+    return false;
+
+  ipPort.SetAddress(ip, port);
+  return true;
+}
+
+
 OpalListener * OpalTransportAddress::CreateListener(OpalEndPoint & endpoint,
                                                     BindOptions option) const
 {
@@ -209,8 +224,17 @@ void OpalTransportAddress::SetInternalTransport(WORD port, const char * proto)
   if (transport == NULL)
     return;
 
-  if (port != 0 && Find(':', dollar) == P_MAX_INDEX)
-    sprintf(":%u", port);
+  // Get port, even if string is bracketed, i.e. udp$[2001...]:1720
+  PINDEX rbracket = Find(']');
+  if( rbracket != P_MAX_INDEX )
+	  dollar = rbracket + 1;
+
+  if (port != 0 && Find(':', dollar) == P_MAX_INDEX) {
+    PINDEX end = GetLength();
+    if (GetAt(end-1) == '+')
+      end--;
+    Splice(psprintf(":%u", port), end);
+  }
 }
 
 
@@ -310,7 +334,7 @@ PString OpalInternalIPTransport::GetHostName(const OpalTransportAddress & addres
 
   PIPSocket::Address ip;
   if (ip.FromString(host))
-    return ip.AsString();
+    return ip.AsString(true);
 
   return host;
 }
@@ -333,7 +357,7 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     port = 0;
   else {
     if (!service) {
-      PString proto = address.Left(address.Find('$'));
+      PCaselessString proto = address.GetProto();
       if (proto *= "ip")
         proto = "tcp";
       port = PIPSocket::GetPortByService(proto, service);
@@ -344,7 +368,11 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     }
   }
 
-  if (host[0] == '*' || host == "0.0.0.0") {
+  if (host[0] == '*' || host == "0.0.0.0"
+#ifdef P_HAS_IPV6
+	  || host == ":" || host == "::"  || host == "[::]"
+#endif
+	  ) {
     ip = PIPSocket::GetDefaultIpAny();
     return PTrue;
   }
@@ -479,85 +507,6 @@ PBoolean OpalListener::StartThread(const PNotifier & theAcceptHandler, ThreadMod
 }
 
 
-//////////////////////////////////////////////////////////////////////////
-
-OpalTransportAddressArray OpalGetInterfaceAddresses(const OpalListenerList & listeners,
-                                                    PBoolean excludeLocalHost,
-                                                    OpalTransport * associatedTransport,
-                                                    PNatMethod * natMethod)
-{
-  OpalTransportAddressArray interfaceAddresses;
-
-  for (OpalListenerList::const_iterator listener = listeners.begin(); listener != listeners.end(); ++listener) {
-    OpalTransportAddressArray newAddrs = OpalGetInterfaceAddresses(listener->GetTransportAddress(), excludeLocalHost, associatedTransport, natMethod);
-    PINDEX size  = interfaceAddresses.GetSize();
-    PINDEX nsize = newAddrs.GetSize();
-    interfaceAddresses.SetSize(size + nsize);
-    PINDEX j;
-    for (j = 0; j < nsize; j++)
-      interfaceAddresses.SetAt(size + j, new OpalTransportAddress(newAddrs[j]));
-  }
-
-  return interfaceAddresses;
-}
-
-
-static void AddTransportAddress(OpalTransportAddressArray & addresses, const PIPSocket::Address & ip, WORD port, PNatMethod * natMethod)
-{
-  addresses.Append(new OpalTransportAddress(ip, port));
-
-  if (natMethod == NULL)
-    return;
-
-  PIPSocket::Address natIP;
-  if (!natMethod->GetInterfaceAddress(natIP))
-    return;
-
-  if (natIP != ip)
-    return;
-
-  if (natMethod->GetExternalAddress(natIP))
-    addresses.Append(new OpalTransportAddress(natIP, port));
-}
-
-
-OpalTransportAddressArray OpalGetInterfaceAddresses(const OpalTransportAddress & addr,
-                                                    PBoolean excludeLocalHost,
-                                                    OpalTransport * associatedTransport,
-                                                    PNatMethod * natMethod)
-{
-  OpalTransportAddressArray interfaceAddresses;
-  PIPSocket::InterfaceTable interfaces;
-
-  PIPSocket::Address ip;
-  WORD port = 0;
-  if (!addr.GetIpAndPort(ip, port) || !ip.IsAny() || !PIPSocket::GetInterfaceTable(interfaces))
-    AddTransportAddress(interfaceAddresses, ip, port, natMethod);
-  else {
-    PINDEX i;
-    PIPSocket::Address firstAddress(0);
-
-    if (associatedTransport != NULL) {
-      if (associatedTransport->GetLocalAddress().GetIpAddress(firstAddress)) {
-        for (i = 0; i < interfaces.GetSize(); i++) {
-          PIPSocket::Address ip = interfaces[i].GetAddress();
-          if (ip == firstAddress)
-            AddTransportAddress(interfaceAddresses, ip, port, natMethod);
-        }
-      }
-    }
-
-    for (i = 0; i < interfaces.GetSize(); i++) {
-      PIPSocket::Address ip = interfaces[i].GetAddress();
-      if (ip != firstAddress && !(excludeLocalHost && ip.IsLoopback()))
-        AddTransportAddress(interfaceAddresses, ip, port, natMethod);
-    }
-  }
-
-  return interfaceAddresses;
-}
-
-
 //////////////////////////////////////////////////////////////////////////
 
 OpalListenerIP::OpalListenerIP(OpalEndPoint & ep,
@@ -581,35 +530,22 @@ OpalListenerIP::OpalListenerIP(OpalEndPoint & endpoint,
 }
 
 
-OpalTransportAddress OpalListenerIP::GetLocalAddress(const OpalTransportAddress & preferredAddress) const
+OpalTransportAddress OpalListenerIP::GetLocalAddress(const OpalTransportAddress & remoteAddress) const
 {
-  PString addr;
+  PIPSocket::Address localIP = localAddress;
 
-  // If specifically bound to interface use that
-  if (!localAddress.IsAny())
-    addr = localAddress.AsString();
-  else {
-    // If bound to all, then use '*' unless a preferred address is specified
-    addr = "*";
-
-    PIPSocket::Address ip;
-    if (preferredAddress.GetIpAddress(ip)) {
-      // Verify preferred address is actually an interface in this machine!
-      PIPSocket::InterfaceTable interfaces;
-      if (PIPSocket::GetInterfaceTable(interfaces)) {
-        for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
-          if (interfaces[i].GetAddress() == ip) {
-            addr = ip.AsString();
-            break;
-          }
-        }
-      }
+  PIPSocket::Address remoteIP;
+  if (remoteAddress.GetIpAddress(remoteIP)) {
+    OpalManager & manager = endpoint.GetManager();
+    PNatMethod * natMethod = manager.GetNatMethod(remoteIP);
+    if (natMethod != NULL) {
+      if (localIP.IsAny())
+        natMethod->GetInterfaceAddress(localIP);
+      manager.TranslateIPAddress(localIP, remoteIP);
     }
   }
 
-  addr.sprintf(":%u", listenerPort);
-
-  return GetProtoPrefix() + addr;
+  return OpalTransportAddress(localIP, listenerPort, GetProtoPrefix());
 }
 
 
@@ -655,27 +591,13 @@ PBoolean OpalListenerTCP::Open(const PNotifier & theAcceptHandler, ThreadMode mo
     return StartThread(theAcceptHandler, mode);
   }
 
-  if (listener.Listen(localAddress, 1, listenerPort))
+  if (listener.Listen(localAddress, 10, listenerPort, exclusiveListener ? PSocket::AddressIsExclusive : PSocket::CanReuseAddress))
     return StartThread(theAcceptHandler, mode);
 
-  if (exclusiveListener) {
-    PTRACE(1, "Listen\tOpen on " << localAddress << ':' << listener.GetPort()
-           << " failed: " << listener.GetErrorText());
-    return PFalse;
-  }
-
-  if (listener.GetErrorNumber() != EADDRINUSE)
-    return PFalse;
-
-  PTRACE(1, "Listen\tSocket for " << localAddress << ':' << listener.GetPort()
-         << " already in use, incoming connections may not all be serviced!");
-
-  if (listener.Listen(localAddress, 100, 0, PSocket::CanReuseAddress))
-    return StartThread(theAcceptHandler, mode);
-
-  PTRACE(1, "Listen\tOpen (REUSEADDR) on " << localAddress << ':' << listener.GetPort()
+  PTRACE(1, "Listen\tOpen (" << (exclusiveListener ? "EXCLUSIVE" : "REUSEADDR") << ") on "
+         << localAddress.AsString(true) << ':' << listener.GetPort()
          << " failed: " << listener.GetErrorText());
-  return PFalse;
+  return false;
 }
 
 
@@ -719,7 +641,6 @@ OpalTransport * OpalListenerTCP::Accept(const PTimeInterval & timeout)
 }
 
 
-
 OpalTransport * OpalListenerTCP::CreateTransport(const OpalTransportAddress & localAddress,
                                                  const OpalTransportAddress & remoteAddress) const
 {
@@ -772,8 +693,14 @@ OpalListenerUDP::~OpalListenerUDP()
 
 PBoolean OpalListenerUDP::Open(const PNotifier & theAcceptHandler, ThreadMode /*mode*/)
 {
-  if (listenerBundle->Open(listenerPort))
-    return StartThread(theAcceptHandler, SingleThreadMode);
+  if (listenerBundle->Open(listenerPort) && StartThread(theAcceptHandler, SingleThreadMode)) {
+    /* UDP packets need to be handled. Not so much at high speed, but must not be
+       significantly delayed by media threads which are running at HighPriority.
+       This, for example, helps make sure that a SIP BYE is received and processed
+       to kill a call where codecs etc in the media threads are hogging all the CPU. */
+    thread->SetPriority(PThread::HighestPriority);
+    return true;
+  }
 
   PTRACE(1, "Listen\tCould not start any UDP listeners");
   return PFalse;
@@ -833,7 +760,7 @@ OpalTransport * OpalListenerUDP::CreateTransport(const OpalTransportAddress & lo
 
   PString iface;
   if (localAddress.GetIpAddress(addr))
-    iface = addr.AsString();
+    iface = addr.AsString(true);
 
   return new OpalTransportUDP(endpoint, PBYTEArray(), listenerBundle, iface, PIPSocket::GetDefaultIpAny(), 0);
 }
@@ -845,6 +772,27 @@ const char * OpalListenerUDP::GetProtoPrefix() const
 }
 
 
+OpalTransportAddress OpalListenerUDP::GetLocalAddress(const OpalTransportAddress & remoteAddress) const
+{
+  PIPSocket::Address localIP = PIPSocket::GetDefaultIpAny();
+  WORD port = listenerPort;
+
+  PIPSocket::Address remoteIP;
+  if (remoteAddress.GetIpAddress(remoteIP)) {
+    PNatMethod * natMethod = endpoint.GetManager().GetNatMethod(remoteIP);
+    if (natMethod != NULL) {
+      natMethod->GetInterfaceAddress(localIP);
+      listenerBundle->GetAddress(localIP.AsString(), localIP, port, true);
+    }
+  }
+
+  if (localIP.IsAny())
+    listenerBundle->GetAddress(PString::Empty(), localIP, port, false);
+
+  return OpalTransportAddress(localIP, port, GetProtoPrefix());
+}
+
+
 //////////////////////////////////////////////////////////////////////////
 
 OpalTransport::OpalTransport(OpalEndPoint & end)
@@ -972,14 +920,14 @@ OpalTransportIP::OpalTransportIP(OpalEndPoint & end,
                                  WORD port)
   : OpalTransport(end),
     localAddress(binding),
-    remoteAddress(0)
+    remoteAddress(PIPSocket::Address::GetAny(binding.GetVersion()))
 {
   localPort = port;
   remotePort = 0;
 }
 
 
-OpalTransportAddress OpalTransportIP::GetLocalAddress() const
+OpalTransportAddress OpalTransportIP::GetLocalAddress(bool /*allowNAT*/) const
 {
   return OpalTransportAddress(localAddress, localPort, GetProtoPrefix());
 }
@@ -1013,6 +961,7 @@ PBoolean OpalTransportIP::SetRemoteAddress(const OpalTransportAddress & address)
   if (IsCompatibleTransport(address))
     return address.GetIpAndPort(remoteAddress, remotePort);
 
+  PTRACE(2, "OpalIP\tAttempt to set incompatible transport " << address);
   return PFalse;
 }
 
@@ -1073,7 +1022,7 @@ PBoolean OpalTransportTCP::Connect()
   WORD firstPort = localPort;
   for (;;) {
     PTRACE(4, "OpalTCP\tConnecting to "
-           << remoteAddress << ':' << remotePort
+           << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
     if (socket->Connect(localPort, remoteAddress))
       break;
@@ -1081,7 +1030,7 @@ PBoolean OpalTransportTCP::Connect()
     int errnum = socket->GetErrorNumber();
     if (localPort == 0 || (errnum != EADDRINUSE && errnum != EADDRNOTAVAIL)) {
       PTRACE(1, "OpalTCP\tCould not connect to "
-                << remoteAddress << ':' << remotePort
+                << remoteAddress.AsString(true) << ':' << remotePort
                 << " (local port=" << localPort << ") - "
                 << socket->GetErrorText() << '(' << errnum << ')');
       return SetErrorValues(socket->GetErrorCode(), errnum);
@@ -1193,8 +1142,8 @@ PBoolean OpalTransportTCP::OnOpen()
 #endif
 
   PTRACE(3, "OpalTCP\tStarted connection to "
-         << remoteAddress << ':' << remotePort
-         << " (if=" << localAddress << ':' << localPort << ')');
+         << remoteAddress.AsString(true) << ':' << remotePort
+         << " (if=" << localAddress.AsString(true) << ':' << localPort << ')');
 
   return PTrue;
 }
@@ -1242,7 +1191,7 @@ OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
   socket->GetLocal(localAddress, localPort, !manager.IsLocalAddress(remoteAddress));
   Open(socket);
 
-  PTRACE(3, "OpalUDP\tBinding to interface: " << localAddress << ':' << localPort);
+  PTRACE(3, "OpalUDP\tBinding to interface: " << localAddress.AsString(true) << ':' << localPort);
 }
 
 
@@ -1272,11 +1221,11 @@ PBoolean OpalTransportUDP::Connect()
     return PFalse;
 
   if (remoteAddress.IsAny() || remoteAddress.IsBroadcast()) {
-    remoteAddress = PIPSocket::Address::GetBroadcast();
+	  remoteAddress = PIPSocket::Address::GetBroadcast(remoteAddress.GetVersion());
     PTRACE(3, "OpalUDP\tBroadcast connect to port " << remotePort);
   }
   else {
-    PTRACE(3, "OpalUDP\tStarted connect to " << remoteAddress << ':' << remotePort);
+    PTRACE(3, "OpalUDP\tStarted connect to " << remoteAddress.AsString(true) << ':' << remotePort);
   }
 
   if (PAssertNULL(writeChannel) == NULL)
@@ -1326,15 +1275,15 @@ bool OpalTransportUDP::SetInterface(const PString & iface)
 }
 
 
-OpalTransportAddress OpalTransportUDP::GetLocalAddress() const
+OpalTransportAddress OpalTransportUDP::GetLocalAddress(bool allowNAT) const
 {
   PMonitoredSocketChannel * socket = (PMonitoredSocketChannel *)readChannel;
   if (socket != NULL) {
     OpalTransportUDP * thisWritable = const_cast<OpalTransportUDP *>(this);
-    socket->GetLocal(thisWritable->localAddress, thisWritable->localPort, !manager.IsLocalAddress(remoteAddress));
+    socket->GetLocal(thisWritable->localAddress, thisWritable->localPort, allowNAT && !manager.IsLocalAddress(remoteAddress));
   }
 
-  return OpalTransportIP::GetLocalAddress();
+  return OpalTransportIP::GetLocalAddress(allowNAT);
 }
 
 
@@ -1560,7 +1509,7 @@ PBoolean OpalTransportTCPS::Connect()
   WORD firstPort = localPort;
   for (;;) {
     PTRACE(4, "OpalTCPS\tConnecting to "
-           << remoteAddress << ':' << remotePort
+           << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
     if (socket->Connect(localPort, remoteAddress))
       break;
@@ -1568,7 +1517,7 @@ PBoolean OpalTransportTCPS::Connect()
     int errnum = socket->GetErrorNumber();
     if (localPort == 0 || (errnum != EADDRINUSE && errnum != EADDRNOTAVAIL)) {
       PTRACE(1, "OpalTCPS\tCould not connect to "
-                << remoteAddress << ':' << remotePort
+                << remoteAddress.AsString(true) << ':' << remotePort
                 << " (local port=" << localPort << ") - "
                 << socket->GetErrorText() << '(' << errnum << ')');
       return SetErrorValues(socket->GetErrorCode(), errnum);
@@ -1634,8 +1583,8 @@ PBoolean OpalTransportTCPS::OnOpen()
 #endif
 
   PTRACE(3, "OpalTCPS\tStarted connection to "
-         << remoteAddress << ':' << remotePort
-         << " (if=" << localAddress << ':' << localPort << ')');
+         << remoteAddress.AsString(true) << ':' << remotePort
+         << " (if=" << localAddress.AsString(true) << ':' << localPort << ')');
 
   return PTrue;
 }
